@@ -20,6 +20,10 @@
 #include <FL/Fl_Progress.H>
 
 
+
+#define IMMAGINE_LEGAMI_SELECTED_SZ 3
+
+
 /**
  *  atomo* root=reinterpret_cast<atomo*>(data);
  * float dx=*(reinterpret_cast<float*>(data2));
@@ -67,8 +71,10 @@ public:
    */
 
   immagine(const immagine& altra);
+  immagine(const immagine* altra);
 
   immagine& operator=(const immagine& altra);
+  immagine& operator=(const immagine* altra);
 
   /**
    *Costruisce, l'oggetto parsando il file passato come argomento.
@@ -103,6 +109,12 @@ public:
   gruppo* find_group_id(int id);
 
 
+  std::vector<gruppo>::iterator first_gruppo();
+
+  std::vector<gruppo>::iterator end_gruppo();
+
+
+
   /**
    *incolla la selezione traslandola
    *
@@ -152,6 +164,25 @@ public:
   bool prova_click(int x, int y, bool& exists, bool erase_previous=true);
   
 
+  /**
+   *check which labels  are under mouse pointer
+   *if any they will be added to _elem_selected
+   * \return true if something has been hitted false otherwise
+   *\param exists true if hitted label already exists
+   */
+  bool check_click_etichetta(int x, int y , bool& exists, bool erase_previous);
+
+
+  /**
+   *check which bond  is under mouse pointer
+   *if any it will be added to _elem_selected
+   * \return true if something has been hitted false otherwise
+   *\param exists true if hitted label already exists
+   */
+  bool check_click_bond(int x, int y , bool& exists, bool erase_previous);
+
+
+
   /**
    *check if there is an element under mouse pointer <b>and</b> if is in
    *current _elem_selected vector.
@@ -162,6 +193,12 @@ public:
 
 
 
+  bool check_selected_group_atom_coincidence(atomo** selected, atomo** hook);
+
+
+
+  bool check_selected_group_bond_coincidence(atomo** selected_1, atomo** selected_2,
+                                             atomo** hook_1    , atomo** hook_2);
 
   /**
    *check ifthere  is a control point  of a procedura  under the mouse
@@ -181,10 +218,12 @@ public:
    *
    *Gli elementi cliccati vengono aggiunti a _elem_selected.
    *I legami cliccati  vengono aggiunti a _legami_selected.
+   *\return true if something has been added
    */
   
-  void click_gruppo(int x, int y);
+  bool click_gruppo(int x, int y, bool erase_previous=false);
 
+  void remove_gruppo_id(int gid);
   
   /**
    *Controlla quali elementi sono ddentro la bounding box.
@@ -290,10 +329,17 @@ public:
    *\param hor true se il flipping deve essere orizzontale false se verticale
    */
 
-
   void flip(int x,int y, bool hor=true);
 
 
+  void flip_group (int group_id, bool hor=true);
+
+  void copy_flip(int x,int y, bool hor=true);
+
+  void copy_flip_group(int id_group , bool hor=true);
+
+
+
   /**
    *Cancella tutta l'immagine ma senza deallocarla, non e' un distuttore.
    */
@@ -439,6 +485,8 @@ public:
   bool cambia_prop_arc(int x , int y);
 
 
+  bool change_prop_box(int x , int y);
+
   /**
    *Apre una finestra di dialogo e  cambia le proprieta dell'atomo a cui
    *appertengono le coordinate date come argomento.
@@ -508,7 +556,11 @@ public:
   void crea_ellisse_nuova(int xc,int yc, int w, int h);
 
 
+  /**
+   *create a new box
+   */
 
+  void create_new_box(int ld_x, int ld_y, int ru_x, int ru_y);
 
 
   /**
@@ -554,7 +606,7 @@ public:
   /**
    *Salva un gruppo nel formato nativo
    */
-  void w_gruppo(int pos,ofstream& out);
+  void w_gruppo(int pos,std::ofstream& out);
  
 
   /**
@@ -649,12 +701,23 @@ public:
 
   unsigned int elem_selected_size();
 
-  //DA LEVAREEEEEE
   void PROVA_ISOMORPH(){
     std::cerr << "isomorph!!" << std::endl;
     _gruppi[0].substructure_match(_gruppi[1]);
   }
   
+  void aabb_elem_selected(pair<float,float>& ld,
+                          pair<float,float>& ru);
+
+
+
+  /**
+   *Aggiunge il gruppo added co n  tutti i suoi legami e procedure agli
+   *elementi selezionati
+   */
+
+  void aggiungi_tutto_selected(gruppo added);
+
 
 protected:
 
@@ -668,13 +731,6 @@ protected:
   void spezza_gruppo();
 
 
-  /**
-   *Aggiunge il gruppo added co n  tutti i suoi legami e procedure agli
-   *elementi selezionati
-   */
-
-  void aggiungi_tutto_selected(gruppo added);
-
 
   /**
    *Cerca una procedura con lo stesso id.
@@ -980,6 +1036,27 @@ protected:
 
 };
 
+
+int glue_find_first_near(atomo* from , atomo* to,
+                         void* dist, void* atom_other, void* res);
+
+/**
+ *\param res_sel_ std::vector< std::pair< std::pair<atomo*,atomo*>, std::pair<float,float> > >
+ */
+
+int glue_find_line_near_sel(atomo* from , atomo* to,
+                            void* res, void* d, void* d1);
+
+/**
+ *\param res_sel_ std::vector< std::pair< std::pair<atomo*,atomo*>, std::pair<float,float> > >
+ *\param res std::vector<unsigned int> where res_sel[res[0]] are pointers to atom in selected group for bond
+ * and res[1] , res[2] are id of atoms in the other group
+ */
+
+int glue_find_line_near_hook(atomo* from , atomo* to,
+                             void* res_sel, void* res, void* found);
+
+
 void export_image_to_vector_file(immagine imm,std::string output_file, bool graphics=true);
 
 
