@@ -17,13 +17,26 @@ public:
   void inizialize();
 
 protected:
+  /**
+   *\param the_group if NULL a new group will be created
+   **/
   void insert_parent_bra(gruppo* the_group,float px,float py,float w,float h);
+  /**
+   *\param the_group if NULL a new group will be created
+   **/
   void insert_box_bra(gruppo* the_group, float px, float py, float w, float h);
+  /**
+   *\param the_group if NULL a new group will be created
+   **/
   void insert_curly_bra(gruppo* the_group,float px,float py,float w,float h);
 
   bool _has_to_act;
   bool _has_acted;
+  bool _merge;
+  bool _free_wrap;
   int  _type;
+  std::vector< std::pair<float,float> > _box;
+
 };
 
 extern "C" bist_plugin* create_plugin(immagine* imm, string libpath);
