@@ -68,7 +68,7 @@
 #include <string_tokenizer.hpp>
 
 
-extern finestra_pr* __la_finestra;
+extern finestra_pr* MainWindow;
 
 extern Preferences  __pref;
 
@@ -671,7 +671,7 @@ void TREcalc::calculateTRE(){
 	double TRE=(E_cyclic - E_acyclic)/num_el_pi;
 	ostringstream TRE_str;
 	TRE_str << "Value of TRE is: " << TRE;
-	fl_message(TRE_str.str().c_str());
+	fl_message("%s",TRE_str.str().c_str());
       }
 
     }
@@ -746,6 +746,11 @@ template <class T>  bool TREcalc::similar_to(T v, T ref, T offset){
   return res;
 }
 
+std::string TREcalc::menu_path(){
+  return "calculation/";
+}
+
+
 /**************fine metodi di classe**********************************/
 
 
