@@ -16,22 +16,6 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-/*
-#include <global.hpp>
-
-#include <FL/fl_draw.H>
-#include <FL/Fl_Double_Window.H>
-#include <FL/Fl_Scroll.H>
-
-
-#include <interfacce.hpp>
-#include <etichetta.hpp>
-
-
-#include <mol_canvas.hpp>
-#include <finestra_pr.hpp>
-*/
-
 
 #include <global.hpp>
 
@@ -70,7 +54,7 @@
 #include <prefs.hpp>
 #include <util.hpp>
 
-extern finestra_pr* __la_finestra;
+extern finestra_pr* MainWindow;
 
 
 
@@ -89,12 +73,12 @@ etichetta::etichetta()
     _y(0),
     _v_offset(0),
     _draw_cursor(false),
-    _draw_negative(false)
+    _draw_negative(false),
+    _can_highlight(false)
 
 {
   _cursor_pos.first=0;
   _cursor_pos.second=0;
-  //aggiungi(NO_ETIC, ET_STR);
 
 }
 
@@ -113,30 +97,11 @@ etichetta::etichetta(const etichetta& altro)
     _v_offset(altro._v_offset),
     _cursor_pos(altro._cursor_pos),
     _draw_cursor(altro._draw_cursor),
-    _draw_negative(altro._draw_negative)
+    _draw_negative(altro._draw_negative),
+    _can_highlight(altro._can_highlight)
     
 {
   genitore::_il_genitore=altro._il_genitore;
-
-//   std::cerr << "COOOPIA!!!!!!!!" << std::endl;
-//   std::cerr << _cr << "<-" << (altro._cr)<< std::endl;
-//   std::cerr << _cg << "<-" << (altro._cg)<< std::endl;
-//   std::cerr << _cb << "<-" << (altro._cb)<< std::endl;
-//   std::cerr << _allineamento << "<-" <<(altro._allineamento)<< std::endl;
-//   std::cerr << _font << "<-"<<  (altro._font)<< std::endl;
-//   std::cerr <<  _dim<< "<-"<<  (altro._dim)<< std::endl;
-//   std::cerr <<   _x<< "<-"<< (altro._x)<< std::endl;
-//   std::cerr <<  _y<< "<-"<<(altro._y)<< std::endl;
-//   std::cerr <<  _v_offset<< "<-"<<(altro._v_offset)<< std::endl;
-
-//   for(unsigned int i=0;i< _vec_str.size();i++){
-//     std::cerr << _vec_str[i].first << " " << _vec_str[i].second << std::endl;
-//     std::cerr << altro._vec_str[i].first << " " << altro._vec_str[i].second << std::endl;
-//   }
-  
-//   std::cerr << sizeof(*this) << " " << sizeof(altro) << std::endl;
-
-
 }
 
 
@@ -154,30 +119,11 @@ etichetta::etichetta(const etichetta* altro)
     _v_offset(altro->_v_offset),
     _cursor_pos(altro->_cursor_pos),
     _draw_cursor(altro->_draw_cursor),
-    _draw_negative(altro->_draw_negative)
+    _draw_negative(altro->_draw_negative),
+    _can_highlight(altro->_can_highlight)
     
 {
   genitore::_il_genitore=altro->_il_genitore;
-
-//   std::cerr << "COOOPIA punt!!!!!!!!" << std::endl;
-//   std::cerr << _cr << "<-" << (altro._cr)<< std::endl;
-//   std::cerr << _cg << "<-" << (altro._cg)<< std::endl;
-//   std::cerr << _cb << "<-" << (altro._cb)<< std::endl;
-//   std::cerr << _allineamento << "<-" <<(altro._allineamento)<< std::endl;
-//   std::cerr << _font << "<-"<<  (altro._font)<< std::endl;
-//   std::cerr <<  _dim<< "<-"<<  (altro._dim)<< std::endl;
-//   std::cerr <<   _x<< "<-"<< (altro._x)<< std::endl;
-//   std::cerr <<  _y<< "<-"<<(altro._y)<< std::endl;
-//   std::cerr <<  _v_offset<< "<-"<<(altro._v_offset)<< std::endl;
-
-//   for(unsigned int i=0;i< _vec_str.size();i++){
-//     std::cerr << _vec_str[i].first << " " << _vec_str[i].second << std::endl;
-//     std::cerr << altro._vec_str[i].first << " " << altro._vec_str[i].second << std::endl;
-//   }
-  
-//   std::cerr << sizeof(*this) << " " << sizeof(altro) << std::endl;
-
-
 }
 
 
@@ -197,6 +143,8 @@ etichetta& etichetta::operator =(const etichetta& altro){
    _cursor_pos=altro._cursor_pos;
    _draw_cursor=altro._draw_cursor;
    _draw_negative=altro._draw_negative;
+   _can_highlight=altro._can_highlight;
+
    genitore::_il_genitore=altro._il_genitore;
 
    return *this;
@@ -204,16 +152,7 @@ etichetta& etichetta::operator =(const etichetta& altro){
 
 
 etichetta::~etichetta(){
-  /*
-#ifdef DEBUG
-  cout << "Eliminata etichetta ";
-
-  for(int i=0;i<vec_str().size();i++){
-    cout << (vec_str())[i].first << " ";
-  }
-  cout << endl;
-#endif
-  */
+  //TODO
 }
 
 
@@ -312,7 +251,6 @@ void etichetta::phys_misura(float& altris,float& larris ){
   
 
   if(cairo_t_singleton::can_export()){
-    //DEBUG_TO_CERR(to_raw_string());
     measure_as_pango_layout(_vec_str,larris,altris);
   }else{
 
@@ -327,32 +265,26 @@ void etichetta::phys_misura(float& altris,float& larris ){
     get_font_handle((*this), tipo_font, font_ps);
     
     fl_font(tipo_font,phys_dim());
-    //fl_measure("o", w_f, altris); //useless?
-    
-    //int sc_apic=SCALE_APICI;
     fl_font(tipo_font,phys_dim());
     
     int font_dim_std=phys_dim();
     int std_desc=fl_descent();
-    
-    //int alt_n=fl_height();
     int xatt=0;
-    int scale_font=SCALE_APICI;
+    float scale_font=SCALE_APICI;
     
     
     for(unsigned int i2=0;i2<_vec_str.size();i2++){
       
       h_f=0;
       w_f=0;
-      //    cout << _vec_str[i2].first << endl;
+
       switch(_vec_str[i2].second){
         
       case ET_STR:
         fl_font(tipo_font,font_dim_std);
         fl_measure(_vec_str[i2].first.c_str(), w_f, h_f);
-        //if(phys_dim()<5){
 	altris=h_f;
-        //}
+
         xatt+=static_cast<int>(rintf(fl_width(_vec_str[i2].first.c_str())));
         break;
       case ET_APICE:
@@ -412,36 +344,24 @@ void etichetta::misura(float& altris,float& larris ){
   //fltk says:
   //void fl_draw(const char *, int x, int y, int w, int h);
   //The string is formatted and aligned *inside* the passed box
-
-
-
-
   
   if(cairo_t_singleton::can_export()){
     phys_misura(altris,larris);
   }else{
     string font_ps;
     int tipo_font;
-
-
     int h_f=0;
     int w_f=0;
 
 
     get_font_handle((*this), tipo_font, font_ps);
-  
-
     fl_font(tipo_font,dim()-3);
-    //fl_measure("o", w_f, altris); //useless
-  
-    //int sc_apic=SCALE_APICI;
     fl_font(tipo_font,dim());
 
     int font_dim_std=dim();
     int std_desc=fl_descent();
-    //int alt_n=fl_height();
     int xatt=0;
-    int scale_font=SCALE_APICI;
+    float scale_font=SCALE_APICI;
  
     for(unsigned int i2=0;i2<_vec_str.size();i2++){
 
@@ -449,15 +369,13 @@ void etichetta::misura(float& altris,float& larris ){
       w_f=0;
 
 
-      //    cout << _vec_str[i2].first << endl;
+
       switch(_vec_str[i2].second){
       
       case ET_STR:
         fl_font(tipo_font,font_dim_std);
         fl_measure(_vec_str[i2].first.c_str(), w_f, h_f);
-        //if(dim()<5){
 	altris=h_f;
-        //}
         xatt+=static_cast<int>(rintf(fl_width(_vec_str[i2].first.c_str())));
         break;
       case ET_APICE:
@@ -502,7 +420,6 @@ void etichetta::misura(float& altris,float& larris ){
       }
     }
 
-    //std::cerr << altris << std::endl;
     altris-=std_desc;
     larris=xatt;
   }
@@ -693,8 +610,6 @@ int etichetta::size_str(){
 void etichetta::aggiungi(string nw, int tipo){
   pair <string,int> tmp_p(nw,tipo);
   _vec_str.push_back(tmp_p);
-  //cout << "@#@#@#" <<(_vec_str.back()).first<< " "
-  // << (_vec_str.back()).second <<endl;
 }
 
 
@@ -705,7 +620,7 @@ void etichetta::melt_similar(int start){
         std::string l=_vec_str[start].first;
         std::string r=_vec_str[start+1].first;
         string lpr=l+r;
-        //std::cerr << "summo: " << l << " ed " << r << std::endl;
+
         _vec_str[start].first=lpr;
         std::vector < std::pair<std::string,int> >::iterator it=_vec_str.begin()+start+1;
         _vec_str.erase(it);
@@ -806,11 +721,6 @@ void etichetta::sostituisci(string cosa, int tipo, int where, bool reset_cur)
     iniz+=where;
     (*iniz).first=cosa;
     (*iniz).second=tipo;
-    /*
-    for(int i=0;i<_vec_str.size();i++){
-      cout << _vec_str[i].first << " " << _vec_str[i].second << endl;
-    }
-    */
 
     if(reset_cur){
       reset_cursor_if_outside_limits();
@@ -945,24 +855,18 @@ void etichetta::disegna(){
 
   get_font_handle((*this), tipo_font, font_ps);
 
-  //int sc_apic=SCALE_APICI;
-
   if(cairo_t_singleton::can_export()){
     draw_label_export();
   }else{
     draw_label_onscreen();
 
-    int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-    int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
+    int dx_scroll=(MainWindow->ritorna_mol_canvas())->x();
+    int dy_scroll=(MainWindow->ritorna_mol_canvas())->y();
 
     if(_draw_cursor){
       int x=-1;
       int y=-1;
       if(cursor_to_coordinate(x,y)){
-	/*
-	std::cerr << "(x+dx_scroll,y+dy_scroll) " << x+dx_scroll << " "<< y+dy_scroll
-		  << std::endl;
-	*/
 	draw_shadowed_point(x+dx_scroll,y+dy_scroll,255,0,0);
       }
 
@@ -978,7 +882,7 @@ void etichetta::measure_as_pango_layout(std::vector < std::pair<std::string,int>
   std::vector<float> ws;
   float hmax=-1;
 
-  unsigned limit_measure_pos=vec.size()-1;
+  unsigned int limit_measure_pos=vec.size()-1;
   if(until>=0 && static_cast<unsigned int>(until)<vec.size() ){
     limit_measure_pos=static_cast<unsigned int>(until);
   }
@@ -991,13 +895,14 @@ void etichetta::measure_as_pango_layout(std::vector < std::pair<std::string,int>
          float wlocal=0;
          float hlocal=0;
          float lbearing;
-         get_pango_logical_sizes(vec[i].first, vec[i].second,wlocal, hlocal,lbearing);
-         if(hlocal>hmax){
+	 float ascent;
+	 float descent;
+         get_pango_logical_sizes(vec[i].first, vec[i].second,wlocal, hlocal,lbearing,ascent,descent);
+
+         if(hlocal>=hmax){
            hmax=hlocal;
          }
-         
          ws.push_back(wlocal);
-         //std::cerr << vec[i].first << " " << wlocal <<std::endl;
        }
        break;
      case ET_APICE:
@@ -1007,16 +912,18 @@ void etichetta::measure_as_pango_layout(std::vector < std::pair<std::string,int>
          float w=0;
          float h_discard=0;
          float lbearing;
-         get_pango_logical_sizes(vec[i].first, vec[i].second,w, h_discard, lbearing);
+	 float ascent;
+	 float descent;
+
+         get_pango_logical_sizes(vec[i].first, vec[i].second,w, h_discard, lbearing,ascent,descent);
          float wlocal=check_pango_max_between_sub_sup(vec,i);
 
-         if(wlocal>0 && wlocal==w){
-           //std::cerr << vec[i].first << " prendo " << w <<std::endl;
-           ws.push_back(wlocal);
-         }else{
-           //std::cerr << "scarto: " <<  vec[i].first << " " 
-           //          << w <<std::endl;
-         }
+
+	 if(wlocal>0 && 
+	    wlocal==w && 
+	    wlocal!=ws.back()){
+	   ws.push_back(wlocal);
+	 }
          
        }
        break;
@@ -1025,16 +932,7 @@ void etichetta::measure_as_pango_layout(std::vector < std::pair<std::string,int>
   }
 
   w=accumulate(ws.begin(),ws.end(),0);
-  //std::cerr << "al finale " << w << std::endl;
   h=hmax;
-  for(unsigned int i=0;i<ws.size();i++){
-    //DEBUG_TO_CERR(ws[i]);
-  }
-
-
-  
-  
-  
 }
 
 
@@ -1059,8 +957,11 @@ float etichetta::check_pango_max_between_sub_sup(std::vector < std::pair<std::st
         float w_logical;
         float h_logical;
         float lbearing;
+	float ascent;
+	float descent;
+
         get_pango_logical_sizes(vec[limit_left].first,vec[limit_left].second,
-                                w_logical,h_logical,lbearing);
+                                w_logical,h_logical,lbearing,ascent,descent);
         sizes.push_back(w_logical);
       }
       limit_left++;
@@ -1100,13 +1001,13 @@ float etichetta::get_fltk_phys_baseline(){
 
 float etichetta::get_pango_baseline(int type){
   int res=-1;
-
+  float scale_font=SCALE_APICI;
   if(_vec_str.size()>0){
     cairo_t* cn=cairo_t_singleton::get_context();
-    int font_dim_std=phys_dim();
+    float font_dim_std=phys_dim();
 
     if(type==ET_APICE || type==ET_PEDICE){
-      font_dim_std/=SCALE_APICI;
+      font_dim_std/=scale_font;
     }
 
     int font_fltk;
@@ -1164,13 +1065,31 @@ float etichetta::get_pango_logical_y(std::string st, int type){
 }
 
 
+float etichetta::get_pango_ink_y(){
+  int res=-1;
+  if(_vec_str.size()>0){
+    res=get_pango_logical_y(_vec_str[0].first, _vec_str[0].second);
+  }
+
+  return res;
+}
+
+float etichetta::get_pango_ink_y(std::string st, int type){
+
+  PangoRectangle rect;
+  get_pango_ink_rectangle(st,type,&rect);
+  return rect.y;
+
+}
+
+
 
 void etichetta::get_pango_logical_rectangle(std::string st, int type, 
                                             PangoRectangle* rect){
 
   cairo_t* cn=cairo_t_singleton::get_context();
-  int font_dim_std=phys_dim();
-  int font_dim_sub=phys_dim()/SCALE_APICI;
+  float font_dim_std=phys_dim();
+  float font_dim_sub=phys_dim()/SCALE_APICI;
 
   int font_fltk;
   std::string font_pango;
@@ -1199,26 +1118,21 @@ void etichetta::get_pango_logical_rectangle(std::string st, int type,
   pango_cairo_update_layout(cn, layout);
   
   
-  pango_layout_get_extents(layout,
-                           rect,
-                           NULL);
-
+  pango_layout_get_extents(layout,NULL,rect);
   
-  pango_extents_to_pixels(rect,
-                          NULL);
+  pango_extents_to_pixels(rect,NULL);
   
 
 
 }
 
 
-void etichetta::get_pango_logical_sizes(std::string st, int type, 
-                                        float& w_logical, float& h_logical, 
-                                        float& lbearing){
+void etichetta::get_pango_ink_rectangle(std::string st, int type, 
+					PangoRectangle* rect){
+
   cairo_t* cn=cairo_t_singleton::get_context();
-  cairo_save(cn);
-  int font_dim_std=phys_dim();
-  int font_dim_sub=phys_dim()/SCALE_APICI;
+  float font_dim_std=phys_dim();
+  float font_dim_sub=phys_dim()/SCALE_APICI;
 
   int font_fltk;
   std::string font_pango;
@@ -1228,6 +1142,121 @@ void etichetta::get_pango_logical_sizes(std::string st, int type,
 
   pango_font_description_set_family(desc, font_pango.c_str() );
 
+  switch(type){
+  case ET_STR:
+  pango_font_description_set_absolute_size(desc,
+                                  font_dim_std*PANGO_SCALE);
+  break;
+  case ET_APICE:
+  case ET_PEDICE:
+    pango_font_description_set_absolute_size(desc,
+                                  font_dim_sub*PANGO_SCALE);
+    break;
+
+  }
+
+  PangoLayout *layout = pango_cairo_create_layout(cn);
+  pango_layout_set_text(layout, st.c_str(), -1);
+  pango_layout_set_font_description(layout, desc);
+  pango_cairo_update_layout(cn, layout);
+  
+  
+  pango_layout_get_extents(layout,rect,NULL);
+  pango_extents_to_pixels(rect,NULL);
+
+}
+
+
+void etichetta::get_pango_logical_sizes(std::string st, int type, 
+                                        float& w_logical, float& h_logical, 
+                                        float& lbearing, float& ascent, float& descent){
+  cairo_t* cn=cairo_t_singleton::get_context();
+  cairo_save(cn);
+  float font_dim_std=phys_dim();
+  float font_dim_sub=phys_dim()/SCALE_APICI;
+
+
+
+  int font_fltk;
+  std::string font_pango;
+  get_font_handle((*this), font_fltk,font_pango);
+
+  PangoFontDescription* desc=pango_font_description_new();
+
+  pango_font_description_set_family(desc, font_pango.c_str());
+
+  switch(type){
+  case ET_STR:
+    pango_font_description_set_absolute_size(desc,
+					     font_dim_std*PANGO_SCALE);
+
+    break;
+  case ET_APICE:
+  case ET_PEDICE:
+    pango_font_description_set_absolute_size(desc,
+					     font_dim_sub*PANGO_SCALE);
+
+    break;
+    
+  }
+
+  PangoLayout *layout = pango_cairo_create_layout(cn);
+  pango_layout_set_text(layout, st.c_str(), -1);
+  pango_layout_set_font_description(layout, desc);
+  pango_cairo_update_layout(cn, layout);
+  PangoRectangle logc_r;
+  pango_layout_get_extents(layout,NULL,&logc_r);
+  pango_extents_to_pixels(&logc_r,NULL);
+
+  
+
+  w_logical=PANGO_RBEARING(logc_r);
+  h_logical=get_pango_baseline(type);
+  descent=PANGO_DESCENT(logc_r);
+  ascent=PANGO_ASCENT(logc_r);
+
+  lbearing=PANGO_LBEARING(logc_r);
+
+
+
+  //manage with ending withespaces
+  
+
+  PangoContext* p_cn=pango_layout_get_context(layout);
+  PangoFont* p_fnt=pango_context_load_font(p_cn,desc);
+  PangoLanguage* def_locale=pango_language_get_default();
+  PangoFontMetrics* p_fnt_metrics=pango_font_get_metrics(p_fnt, def_locale);
+  float  char_width=pango_font_metrics_get_approximate_char_width(p_fnt_metrics) / PANGO_SCALE;
+
+  
+  for(int i=st.size()-1; i>=0  && isblank(st[i]);i--){
+    w_logical+=char_width;
+  }
+  
+  g_object_unref(layout);
+  pango_font_description_free(desc);
+
+  cairo_restore(cn);
+
+}
+
+
+void etichetta::get_pango_ink_sizes(std::string st, int type, 
+				    float& w_ink, float& h_ink, 
+				    float& lbearing, float& ascent, float& descent){
+  cairo_t* cn=cairo_t_singleton::get_context();
+  cairo_save(cn);
+  float font_dim_std=phys_dim();
+  float font_dim_sub=phys_dim()/SCALE_APICI;
+
+  int font_fltk;
+  std::string font_pango;
+  get_font_handle((*this), font_fltk,font_pango);
+
+  PangoFontDescription* desc=pango_font_description_new();
+
+  pango_font_description_set_family(desc, font_pango.c_str());
+
   switch(type){
   case ET_STR:
     pango_font_description_set_absolute_size(desc,
@@ -1247,28 +1276,17 @@ void etichetta::get_pango_logical_sizes(std::string st, int type,
   pango_layout_set_text(layout, st.c_str(), -1);
   pango_layout_set_font_description(layout, desc);
   pango_cairo_update_layout(cn, layout);
-  PangoRectangle logc_r;
+  PangoRectangle ink_r;
   
-  pango_layout_get_extents(layout,
-                           &logc_r,
-                           NULL);
+  pango_layout_get_extents(layout,&ink_r,NULL);
+  pango_extents_to_pixels(&ink_r,NULL);
 
-  
-  pango_extents_to_pixels(&logc_r,
-                          NULL);
-  
-
-  
-
-  //pango_layout_get_size(layout,&w_logical,&h_logical);
-
-  //std::cerr << "logc_r: " << logc_r.width << " " << logc_r.x<< " " << std::endl;
-  w_logical=PANGO_RBEARING(logc_r);
-  h_logical=get_pango_baseline(type);
-
-
-  lbearing=PANGO_LBEARING(logc_r);
+  w_ink=PANGO_RBEARING(ink_r);
+  h_ink=get_pango_baseline(type);
+  descent=PANGO_DESCENT(ink_r);
+  ascent=PANGO_ASCENT(ink_r);
 
+  lbearing=PANGO_LBEARING(ink_r);
 
   //manage with ending withespaces
   
@@ -1280,16 +1298,13 @@ void etichetta::get_pango_logical_sizes(std::string st, int type,
   float  char_width=pango_font_metrics_get_approximate_char_width(p_fnt_metrics) / PANGO_SCALE;
 
   
-  for(unsigned int i=st.size()-1;isblank(st[i]);i--){
-    w_logical+=char_width;
+  for(int i=st.size()-1; i>=0  && isblank(st[i]);i--){
+    w_ink+=char_width;
   }
   
   g_object_unref(layout);
   pango_font_description_free(desc);
 
-  //w_logical/=PANGO_SCALE;
-  //h_logical/=PANGO_SCALE;
-
   cairo_restore(cn);
 
 }
@@ -1301,24 +1316,27 @@ void etichetta::draw_pango_sub_superscript(std::string st, int hparent,
 
   bool typeok=false;
   int nwy=y;
+  float w_logical;
+  float h_logical;
+  float lbearing;
+  float ascent;
+  float descent;
+
 
   if(type==ET_APICE){
     typeok=true;
-    
+    get_pango_logical_sizes(st, type, w_logical,  h_logical, lbearing,ascent, descent);
+    nwy-=hparent/2.0;
+    nwy+=h_logical;
   }else if(type==ET_PEDICE){
-    //std::cerr << st << " e' pedice " << hparent << std::endl;
     typeok=true;
-    float w_logical;
-    float h_logical;
-    float lbearing;
-    get_pango_logical_sizes(st, type, w_logical,  h_logical, lbearing);
-    float l_y=get_pango_logical_y(st,type);
-
-
-    nwy+=hparent - l_y - h_logical/ 2;
+    get_pango_logical_sizes(st, type, w_logical,  h_logical, lbearing,ascent, descent);
+    nwy+=hparent;
+    nwy+=- h_logical/2.0;
   }
-  
+ 
   if(typeok){
+
     draw_pango_string(st, x, nwy, type);
   }
 
@@ -1328,7 +1346,7 @@ void etichetta::draw_pango_sub_superscript(std::string st, int hparent,
 void etichetta::draw_pango_string(std::string st,int x, int y, int type){
   cairo_t* cn=cairo_t_singleton::get_context();
   int font_dim_std=phys_dim();
-  int font_dim_sub=phys_dim()/SCALE_APICI;
+  float font_dim_sub=phys_dim()/SCALE_APICI;
 
 
   int font_fltk=0;
@@ -1388,16 +1406,15 @@ void etichetta::draw_label_export(){
       measure_as_pango_layout(_vec_str,
                               wlocal,hlocal,i);
 
-      cairo_set_source_rgb(cn, remap_color_1(cr()),
+      cairo_set_source_rgb(cn,remap_color_1(cr()),
                            remap_color_1(cg()),
                            remap_color_1(cb()));
       
-      //std::cerr << "disegno in " << i << " " <<  x_att<< std::endl;
 
       int threshold=0;
       if( (i+1) < _vec_str.size()){
         draw_pango_sub_superscript(_vec_str[i+1].first, hlocal,
-                                   phys_x()+wlocal,y_att, _vec_str[i+1].second);
+                                   phys_x()+wlocal,y_att,_vec_str[i+1].second);
         
         if(_vec_str[i+1].second==ET_APICE ||
            _vec_str[i+1].second==ET_PEDICE){
@@ -1420,11 +1437,6 @@ void etichetta::draw_label_export(){
       x_att=phys_x()+wlocal;
     }
   }
-
-    
-  // cairo_move_to(cn,phys_x(),phys_y());
-  // cairo_set_line_width (cn, 0.1);
-  // cairo_rectangle(cn,phys_x(),phys_y(),w(),h());
   
   cairo_stroke(cn);
   cairo_restore(cn);
@@ -1438,14 +1450,12 @@ void etichetta::draw_label_onscreen(){
 
   int font_dim_std=visual_dim();
     
+  float xatt=visual_x();
+  float scale_font=SCALE_APICI;
 
-
-
-  int xatt=static_cast<int>(rintf(visual_x()));
-  int scale_font=SCALE_APICI;
-
-  int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-  int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
+  int dx_scroll=(MainWindow->ritorna_mol_canvas())->x();
+  int dy_scroll=(MainWindow->ritorna_mol_canvas())->y();
+  float h_f_parent=0;  
 
   if(draw_negative()){
     fl_color(FL_BLACK);
@@ -1461,14 +1471,12 @@ void etichetta::draw_label_onscreen(){
   }
 
   fl_font(tipo_font,visual_dim());
-  int alt_n=fl_height();
+  
   
   for(unsigned int i2=0;i2<_vec_str.size();i2++){
     int w_f=0;
     int h_f=0;
 
-
-    //    cout << _vec_str[i2].first << endl;
     switch(_vec_str[i2].second){
 
     case ET_STR:
@@ -1479,14 +1487,15 @@ void etichetta::draw_label_onscreen(){
               static_cast<int>(rintf(visual_y()+dy_scroll)),
               w_f,
               h_f, FL_ALIGN_CENTER);
-      xatt+=static_cast<int>(rintf(fl_width(_vec_str[i2].first.c_str())));
+      xatt+=fl_width(_vec_str[i2].first.c_str());
+      h_f_parent=h_f;
       break;
     case ET_APICE:
       fl_font(tipo_font,font_dim_std/scale_font);
       fl_measure(_vec_str[i2].first.c_str(), w_f, h_f);
       fl_draw(_vec_str[i2].first.c_str(),
               xatt+dx_scroll,
-              static_cast<int>(rintf(visual_y()+dy_scroll)),
+              static_cast<int>(rintf(-h_f/2 + visual_y()+dy_scroll)),
               w_f,
               h_f, FL_ALIGN_CENTER);
       if((i2+1)<_vec_str.size()){
@@ -1494,9 +1503,9 @@ void etichetta::draw_label_onscreen(){
           if(_vec_str[i2-1].second==ET_PEDICE){
             float max_l=max( fl_width(_vec_str[i2].first.c_str()) , 
                              fl_width(_vec_str[i2-1].first.c_str()));
-            xatt+=static_cast<int>(rintf(max_l));
+            xatt+=rintf(max_l);
           }else{
-            xatt+=static_cast<int>(rintf(fl_width(_vec_str[i2].first.c_str())));
+            xatt+=fl_width(_vec_str[i2].first.c_str());
           }
         }
       }
@@ -1509,7 +1518,7 @@ void etichetta::draw_label_onscreen(){
       
       fl_draw(_vec_str[i2].first.c_str(),
               xatt+dx_scroll,
-              static_cast<int>(rintf(visual_y()+alt_n/2+dy_scroll)),
+              static_cast<int>(rintf(visual_y()+h_f_parent-h_f/2.0+dy_scroll)),
               w_f,
               h_f, FL_ALIGN_CENTER);
       
@@ -1519,9 +1528,9 @@ void etichetta::draw_label_onscreen(){
           if(_vec_str[i2-1].second==ET_APICE){
             float max_l=max( fl_width(_vec_str[i2].first.c_str()) , 
                              fl_width(_vec_str[i2-1].first.c_str()));
-            xatt+=static_cast<int>(rintf(max_l));
+            xatt+=rintf(max_l);
           }else{
-            xatt+=static_cast<int>(rintf(fl_width(_vec_str[i2].first.c_str())));
+            xatt+=fl_width(_vec_str[i2].first.c_str());
           }
         }
 	
@@ -1553,8 +1562,7 @@ void etichetta::scale(float sc){
   }else{
     dim(phys_dim()+1);
   }
-  
-  //cout << "nuova dim: " << phys_dim() << " " << dim()<< endl;
+
 }
 
 /**
@@ -1568,12 +1576,6 @@ void etichetta::scale(float sc){
 
 void etichetta::ruota(float xpiv, float ypiv,float angl){
   not_impl();
-  /*
-  xpivot(xpiv);
-  ypivot(ypiv);
-  angolorot(angl);
-  */
-  //  draw_rotated(const char* text, int n, int x, int y, int angle);
 }
 
 /**
@@ -1584,8 +1586,6 @@ void etichetta::ruota(float xpiv, float ypiv,float angl){
  */
 
 void etichetta::trasla(float dx, float dy){
-  //phys_translate(dx, dy);
-    
    x(descale(x()+dx));
    y(descale(y()+dy));
   
@@ -1667,13 +1667,7 @@ int etichetta::get_cursor_position(std::pair<int,string::size_type>* pos){
 
   bool stop=false;
   while(!stop){
-    /*
-    std::cerr << to_raw_string() << std::endl;
-    std::cerr << "saved =" << saved_curr_pos.first << "," << saved_curr_pos.second
-              << std::endl;
-    std::cerr << "now =" <<_cursor_pos.first << "," << _cursor_pos.second
-              << std::endl;
-    */
+
     if(_cursor_pos.first==saved_curr_pos.first){
       if(_cursor_pos.second==saved_curr_pos.second){
         stop=true;
@@ -1734,18 +1728,13 @@ bool etichetta::cursor_to_coordinate(int& x, int& y){
     if(_cursor_pos.second<=(the_vector_str.back().first).size()){
       if(_cursor_pos.second==(the_vector_str.back().first).size()){
         //draw here the cursor after the last char  of the label
-
+	//TODO
       }else{
 	std::string to_update=the_vector_str.back().first;
 	to_update=to_update.substr(0,_cursor_pos.second);
         
         std::pair<int,string::size_type> c_pos(0,0);
         copy_et.get_cursor_position(&c_pos);
-
-        /*
-        std::cerr << "raw: " << copy_et.to_raw_string() << std::endl;
-	std::cerr << c_pos.first << " " << c_pos.second << std::endl;
-        */
 	copy_et.sostituisci(to_update,type_str,c_pos.first);
       }
 
@@ -1753,10 +1742,22 @@ bool etichetta::cursor_to_coordinate(int& x, int& y){
       x=copy_et.x()+copy_et.w();
       y=copy_et.y();
 
-      //std::cerr << "y=" << y << " " << copy_et.h() << std::endl;
+      int tipo_font;
+      string font_ps;
+      int h_f=0;
+      int w_f=0;
+      vector < pair<string,int> > vec_str=copy_et.vec_str();
 
-      if(the_vector_str.back().second==ET_PEDICE ||
-	 the_vector_str.back().second==ET_STR){
+      get_font_handle((*this), tipo_font, font_ps);
+      fl_font(tipo_font,dim()/SCALE_APICI);
+      fl_measure(vec_str[0].first.c_str(), w_f, h_f);
+      fl_font(tipo_font,dim());
+
+      if(the_vector_str.back().second==ET_PEDICE){
+	y+=copy_et.h()+h_f;
+      }else if(the_vector_str.back().second==ET_APICE){
+	y+=h_f/SCALE_APICI;
+      }else if(the_vector_str.back().second==ET_STR){
         if(copy_et.h()>=0){
           y+=copy_et.h();
         }else{
@@ -1786,8 +1787,6 @@ bool etichetta::cursor_one_step_fwd(int &curr_type){
   }else{
     res=false;
   }
-  //std::cerr << "car == ->" << std::endl;
-  
 
   curr_type=the_vec_str[_cursor_pos.first].second;
 
@@ -1807,7 +1806,6 @@ bool etichetta::cursor_one_step_back(int &curr_type){
   }else{
     res=false;
   }
-  //cerr << "fermo" <<  std::endl;
   
   curr_type=the_vec_str[_cursor_pos.first].second;
   return res;
@@ -1819,7 +1817,7 @@ bool etichetta::cursor_one_step_back(int &curr_type){
 bool etichetta::check_if_type_etic(int typ) throw (out_of_range){
   bool res=false;
   if(size_str()>0){
-    //int nis=_str->size_str();
+
     vector < pair<string,int> > strings=vec_str();
     pair<string,int> pt= strings.back();
     
@@ -1839,14 +1837,15 @@ bool etichetta::check_if_type_etic(int typ) throw (out_of_range){
 
 
 bool etichetta::check_if_type_etic(int typ,int pos) throw (out_of_range){
- 
-  //int nis=_str->size_str();
+
   vector < pair<string,int> > stringhe=vec_str();
 
   if(static_cast<unsigned int>(pos) < vec_str().size() &&
      pos >=0){
+
     vector < pair<string,int> >::iterator it= stringhe.begin();
     pair<string,int> pt= *(it+pos);
+
     if(pt.second==typ){
       return true;
     }else{
@@ -1861,9 +1860,8 @@ bool etichetta::check_if_type_etic(int typ,int pos) throw (out_of_range){
 
 
 bool etichetta::check_if_type_etic_previous_chunk(int equal,bool &res_prev_exist){
-  
-  vector < pair<string,int> > stringhe=vec_str();
 
+  vector < pair<string,int> > stringhe=vec_str();
   bool res=false;
 
 
@@ -1928,7 +1926,7 @@ void etichetta::manage_sub_super(int check, int troublesome,std::string car){
   //poniamo sia un apice
   try{
     if(check_if_type_etic(check,_cursor_pos.first)){ //ultimo e' apice aggiungere
-      //std::cerr << "ultimo apice!" << std::endl;
+
       for(unsigned int i=0;i<car.size();i++){
         insert_char_in_curr_pos(car[i],check);
       }
@@ -1962,7 +1960,7 @@ void etichetta::manage_sub_super(int check, int troublesome,std::string car){
 }
 
 void etichetta::insert_string_in_curr_pos(std::string add,int type){
-  //std::cerr << "try" << std::endl;
+
   if(type==ET_APICE){
     manage_sub_super(ET_APICE, ET_PEDICE,add);
   }else if(type==ET_PEDICE){//dovremmo aggiungere un pedice
@@ -1976,7 +1974,6 @@ void etichetta::insert_string_in_curr_pos(std::string add,int type){
       
     }catch (out_of_range e){ // il  vettore  e' vuoto  possiamo
       // aggiungere e inizializzarlo
-      //std::cerr << "catch" << std::endl;
       for(unsigned int i=0;i<add.size();i++){
         insert_char_in_curr_pos(add[i],ET_STR);
       }
@@ -1989,6 +1986,7 @@ bool etichetta::manage_add_sub_next_super(int type){
   std::string str=the_vec_str[_cursor_pos.first].first;
   bool res=true;
   bool next_or_prev_exist=false;
+
   if((type_etic_at_cur_pos() >0  && type_etic_at_cur_pos()!=type)){
     bool exist=false;
     if(check_if_type_etic_next_chunk(type,exist)){
@@ -2071,35 +2069,31 @@ bool etichetta::manage_add_sub_super_next_normal_str(int type){
 
 
 void etichetta::insert_char_in_curr_pos(char add, int type){
-  //std::cerr << "OK" << "uso interna " << std::endl;
+
   std::vector < std::pair<std::string,int> > the_vec_str=vec_str();
   if(the_vec_str.size()<=0){ //inizializza il vettore
-    //std::cerr << "//inizializza il vettore" << std::endl;
+
     aggiungi(std::string(1,add),type);
     _cursor_pos.second=1;
     _cursor_pos.first=0;
   }else if(static_cast<unsigned int>(_cursor_pos.first)<the_vec_str.size()){ //posizione first valida
-    //std::cerr << "//posizione first valida" << std::endl;
+
     if(_cursor_pos.second<=0){
-      /*
-      cerr << "entro in " << _pos.first << " " << the_vec_str[_pos.first].first
-	   << " " << _pos.second  << std::endl;
-      */
+
+
       if( the_vec_str[_cursor_pos.first].first.size()>0){
 	std::string to_update= the_vec_str[_cursor_pos.first].first;
 	to_update.insert(_cursor_pos.second,std::string(1,add));
 	sostituisci(to_update,type,_cursor_pos.first);
 	_cursor_pos.second++;
-        //std::cerr << "_cursor_pos.second" << _cursor_pos.second << std::endl;
+
       }else{// ma vuota
-        //std::cerr << "// ma vuota " << std::endl;
 	sostituisci(std::string(1,add),type,_cursor_pos.first);
 	_cursor_pos.second=1;
       }
     }else{ // non vuota inserisci
       std::string to_update= the_vec_str[_cursor_pos.first].first;
-      //std::cerr << "to_update.size()" <<  to_update.size() 
-      //          << " " << _cursor_pos.second << std::endl;
+
       if(_cursor_pos.second<to_update.size()){ //cursore non  alla   fine  della
 					//stringa
 	to_update.insert(_cursor_pos.second,std::string(1,add));
@@ -2109,7 +2103,6 @@ void etichetta::insert_char_in_curr_pos(char add, int type){
 	to_update+=std::string(1,add);
 	sostituisci(to_update,type,_cursor_pos.first);
 	_cursor_pos.second=to_update.size();
-	//std::cerr << "dopo :" <<  to_update.size() << " " << _cursor_pos.second << std::endl;
       }
     }
   }
@@ -2119,7 +2112,7 @@ void etichetta::insert_char_in_curr_pos(char add, int type){
 
 
 void etichetta::delete_char_curr_pos(bool before){
-  //std::cerr << "uso quella interna ad etichetta" << std::endl;
+
   if(size_str()>0){
     std::vector < std::pair<std::string,int> > the_vec_str=vec_str();
     try{
@@ -2132,7 +2125,7 @@ void etichetta::delete_char_curr_pos(bool before){
       if(before){
         if(_cursor_pos.second<=to_update.size() && 
            _cursor_pos.second>0){
-          //std::cerr << "eccoci " << _cursor_pos.second << std::endl;
+
           to_update.erase(_cursor_pos.second-1,1);
           
           if(to_update!=""){
@@ -2153,8 +2146,6 @@ void etichetta::delete_char_curr_pos(bool before){
             int dummy=0;
             cursor_one_step_back(dummy);
           }
-        }else{
-          //cerr << "fermo" <<  std::endl;
         }
       }else{
         if(_cursor_pos.second<to_update.size()){ //cursore non  alla   fine  della
@@ -2288,11 +2279,7 @@ void etichetta::break_normal_str(int type_to_insert){
 
      rem_me.erase(_cursor_pos.second);
      new_chunk_me=new_chunk_me.substr(_cursor_pos.second);
-     /*
-     std::cerr << "rem_me='" << rem_me << "'" << std::endl
-               << "new_chunk_me='" << new_chunk_me << "'" << std::endl
-               << _cursor_pos.first << std::endl;
-     */
+
      elimina(_cursor_pos.first,false);
 
      if(rem_me!=""){
@@ -2318,7 +2305,7 @@ void etichetta::break_normal_str(int type_to_insert){
    if((me==ET_APICE || me==ET_PEDICE) &&
       type_to_insert == ET_STR ){
      int other=-1;
-     //std::cerr << "_cursor_pos.first " << _cursor_pos.first << std::endl;
+
 
      if(me==ET_APICE){
        other=ET_PEDICE;
@@ -2373,12 +2360,7 @@ void etichetta::break_normal_str(int type_to_insert){
        }else{
          new_chunk_other="";
        }
-       /*
-       std::cerr << "rem_me='" << rem_me << "'" << std::endl
-                 << "rem_other='"   << rem_other << "'" << std::endl
-                 << "new_chunk_me='" << new_chunk_me << "'" << std::endl
-                 << "new_chunk_other='"   << new_chunk_other << "'" << std::endl;
-       */
+
     
        elimina(_cursor_pos.first,false);
 
@@ -2420,14 +2402,27 @@ void etichetta::break_normal_str(int type_to_insert){
  }
 
 
- void etichetta::draw_negative(bool nw){
-   _draw_negative=nw;
- }
+void etichetta::draw_negative(bool nw){
+  _draw_negative=nw;
+}
 
 
- bool etichetta::draw_negative(){
-   return _draw_negative;
- }
+bool etichetta::draw_negative(){
+  return _draw_negative;
+}
+
+
+
+bool etichetta::can_highlight(){
+  return _can_highlight;
+}
+
+bool etichetta::can_highlight(bool nw){
+  bool old=_can_highlight;
+  _can_highlight=nw;
+  return old;
+}
+
 
  int etichetta::switch_APIC_PEDIC(int what){
    int res=ET_APICE;
@@ -2474,7 +2469,7 @@ bool etichetta::merge_with_previous_if_equal(bool reset_cur) throw (out_of_range
 
 void etichetta::merge_orphan_pedic_apic(int from, int type,bool& has_merged){
   std::vector < std::pair<std::string,int> > the_vec_str=vec_str();
-  //std::cerr << "from " << from << std::endl;
+
   if(from >=0 && from < static_cast<int>(the_vec_str.size()-1)){
     if(check_if_type_etic(type,from)){
       std::string str_to_merge=the_vec_str[from].first;
@@ -2491,7 +2486,7 @@ void etichetta::merge_orphan_pedic_apic(int from, int type,bool& has_merged){
       }
       sostituisci(str_to_merge,type,from,true); 
       has_merged=true;
-      //std::cerr << "str: " << str_to_merge << std::endl;
+
     }else{
       merge_orphan_pedic_apic(from+1,type,has_merged);
     }
@@ -2502,21 +2497,17 @@ void etichetta::delete_orphan_pedic_apic(int from,int type){
   std::vector < std::pair<std::string,int> > the_vec_str=vec_str();
   if(from >=0 && from < static_cast<int>(the_vec_str.size())){
     for(int i=from;i< static_cast<int>(the_vec_str.size());i++){
-      //std::cerr << "i: " << i << std::endl;
+
       if(check_if_type_etic(type,i)){
         try{
           if(!check_if_type_etic(ET_STR,i-1) &&
              !check_if_type_etic(ET_STR,i-2) ){
             elimina(i, false);
-            //std::cerr << "eliminata: " << the_vec_str[i].first << std::endl;
             the_vec_str=vec_str();
             i--;
-            //std::cerr << "i sceso a: " << i << std::endl;
           }
         }catch (out_of_range e){
-          //std::cerr << "fuori" <<  i << std::endl;
           elimina(i, false);
-          //std::cerr << "eliminata: " << the_vec_str[i].first << std::endl;
           the_vec_str=vec_str();
           i--;
         }
@@ -2529,7 +2520,7 @@ void etichetta::delete_orphan_pedic_apic(int from,int type){
  void etichetta::clean_string_after_delete(){
    std::vector < std::pair<std::string,int> > the_vec_str=vec_str();
    int old_type=type_etic_at_cur_pos();
-   //int old_pos=_cursor_pos.first;
+
    bool old_is_last =false;
    if(_cursor_pos.first == static_cast<int>(the_vec_str.size()-1)){
      old_is_last=true;
@@ -2545,7 +2536,7 @@ void etichetta::delete_orphan_pedic_apic(int from,int type){
    reset_cursor_if_outside_limits(true);
    
    the_vec_str=vec_str();
-   //std::cerr << _cursor_pos.first << std::endl;
+
    if(old_type==ET_APICE || old_type==ET_PEDICE){
 
      if(_cursor_pos.first<static_cast<int>(the_vec_str.size()-1)){
