@@ -16,9 +16,10 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-#include <interfacce.hpp>
-#include <iostream>
 
+#include <iostream>
+#include <string>
+#include <interfacce.hpp>
 
 genitore::genitore()
   :_il_genitore(0)
@@ -55,3 +56,19 @@ void genitore::aggiungi_genitore(genitore* gtr){
 genitore* genitore::ritorna_genitore(){
   return _il_genitore;
 }
+
+
+
+damaged_obj::damaged_obj()
+  :_damaged(false)
+{
+
+}
+  
+bool damaged_obj::damaged(){
+  return _damaged;
+}
+
+void damaged_obj::damaged(bool is_damaged=true){
+  _damaged=is_damaged;
+}
