@@ -69,7 +69,7 @@
 #include <prefs.hpp>
 #include <util.hpp>
 
-extern finestra_pr* __la_finestra;
+extern finestra_pr* MainWindow;
 
 
 
@@ -304,12 +304,11 @@ int multiline_label::font(){
 
   if(!treat_as_a_whole()){
     if(ptr_line_less_lines_text()){
-      _lines_of_text[_pointer_to_currline]->font();
+      res=_lines_of_text[_pointer_to_currline]->font();
     }
   }
   
   return res;
-
 }
 
 
@@ -974,8 +973,8 @@ void multiline_label::disegna(){
         //                          _lines_of_text[i]->get_fltk_baseline()-max_desc );
       }
       /*
-      int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-      int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
+      int dx_scroll=(MainWindow->ritorna_mol_canvas())->x();
+      int dy_scroll=(MainWindow->ritorna_mol_canvas())->y();
 
       fl_rect(_lines_of_text[i]->x()+dx_scroll,
               _lines_of_text[i]->y()+dy_scroll,
@@ -990,8 +989,8 @@ void multiline_label::disegna(){
     */
     
 
-    //int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-    // int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
+    //int dx_scroll=(MainWindow->ritorna_mol_canvas())->x();
+    // int dy_scroll=(MainWindow->ritorna_mol_canvas())->y();
 
 
     old_t=treat_as_a_whole(true);
