@@ -71,7 +71,7 @@
 #include <prefs.hpp>
 #include <util.hpp>
 
-extern finestra_pr* __la_finestra;
+extern finestra_pr* MainWindow;
 
 
 
@@ -94,8 +94,9 @@ paragraph_text::paragraph_text(const paragraph_text& other)
   _pointer_to_currline(other._pointer_to_currline),
   _interline_space(other._interline_space),
   _layout_lines(other._layout_lines)
+
 {
-  //std::cerr << "ricopio paragraph " << __FUNCTION__ << std::endl;
+  
   for(unsigned int i=0;i< (other._lines_of_text.size()) ;i++){
     multifont_label* tmp=new multifont_label(other._lines_of_text[i]);
     _lines_of_text.push_back(tmp);
@@ -108,6 +109,7 @@ paragraph_text::paragraph_text(const paragraph_text* other)
    _pointer_to_currline(other->_pointer_to_currline),
    _interline_space(other->_interline_space),
    _layout_lines(other->_layout_lines)
+
 {
 
   //std::cerr << "ricopio paragraph " << __FUNCTION__ << std::endl;
@@ -120,11 +122,11 @@ paragraph_text::paragraph_text(const paragraph_text* other)
 
 
 paragraph_text& paragraph_text::operator=(const paragraph_text& other){
-   std::cerr << __FUNCTION__ << std::endl;
-   _pointer_to_currline=other._pointer_to_currline;
+  _pointer_to_currline=other._pointer_to_currline;
   _interline_space=other._interline_space;
   _layout_lines=other._layout_lines;
 
+
   for(unsigned int i=0;i< (other._lines_of_text.size()) ;i++){
     multifont_label* tmp=new multifont_label(other._lines_of_text[i]);
     _lines_of_text.push_back(tmp);
@@ -374,29 +376,62 @@ void paragraph_text::dehighlight_current_line(){
 }
 
 
+
+bool paragraph_text::highlight_under_bounding_box(std::pair<float,float> ld,
+						  std::pair<float,float> ru){
+  
+  format_paragraph();
+  bool res=false;
+  for(unsigned int i=0; i<_lines_of_text.size(); i++){
+    std::vector<etichetta*> labels=_lines_of_text[i]->get_lines_text();
+    for(unsigned int j=0; j<labels.size(); j++){
+
+      if(labels[j]->x() > ld.first  && labels[j]->x()  < ru.first &&
+	 labels[j]->y() > ld.second && labels[j]->y()  < ru.second){
+	labels[j]->draw_negative(true);
+	res=true;
+      }
+    }
+  }
+  
+  return res;
+}
+
+bool paragraph_text::some_highlighted(){
+  
+  bool res=false;
+  for(unsigned int i=0; i<_lines_of_text.size(); i++){
+    std::vector<etichetta*> labels=_lines_of_text[i]->get_lines_text();
+    for(unsigned int j=0; j<labels.size(); j++){
+      if(labels[j]->draw_negative()){
+	res=true;
+	break;
+      }
+    }
+  }
+  
+  return res;
+}
+
+
+
+
 bool paragraph_text::invert_highlight_child_and_shift_cursor(bool fwd){
   int dummy=0;
   bool res=false;
   if(fwd){
-    int oldline=pointer_to_currline();
+    pointer_to_currline();
     cursor_one_step_fwd(dummy);
+    _lines_of_text[pointer_to_currline()]->go_to_end_of_line();
     res=highlight_child(true);
     highlight_child(!res);
-    if(oldline < pointer_to_currline()){
-      cursor_one_step_fwd(dummy);
-    }
 
   }else{
-    int oldline=pointer_to_currline();
+    pointer_to_currline();
     res=highlight_child(true);
     highlight_child(!res);
+    _lines_of_text[pointer_to_currline()]->go_to_start_of_line();
     cursor_one_step_back(dummy);
-    if(oldline > pointer_to_currline()){
-      cursor_one_step_fwd(dummy);
-      res=highlight_child(true);
-      highlight_child(!res);
-      cursor_one_step_back(dummy);
-    }
   }
   
   return res;
@@ -545,15 +580,60 @@ void paragraph_text::color_child(int cr, int cg, int cb,
     _lines_of_text[pointer_to_currline()]->pointer_to_currline(saved_ptr_line_child);
     _lines_of_text[act_ptr_line]->treat_as_a_whole(old_t);
   }
+}
 
 
 
+void paragraph_text::query_child(int* cr, int* cg, int* cb,
+                                 int* font, int* dim,
+                                 int* ptr_line, int* ptr_line_child){
+
+
+
+  int saved_ptr_line=-1;
+  int saved_ptr_line_child=-1;
+  int act_ptr_line=-1;
+  int act_ptr_line_child=-1;
+
+  bool is_legal=set_ptrline_to_modify_children(ptr_line, 
+                                               ptr_line_child,
+                                               &saved_ptr_line, 
+                                               &saved_ptr_line_child,
+                                               &act_ptr_line, 
+                                               &act_ptr_line_child);
+
+  if(is_legal){
+    bool old_t=_lines_of_text[act_ptr_line]->treat_as_a_whole(false);
+    _lines_of_text[act_ptr_line]->pointer_to_currline(act_ptr_line_child);
+    if(cr!=NULL){
+      *cr=_lines_of_text[act_ptr_line]->cr();
+    }
+    if(cg!=NULL){
+      *cg=_lines_of_text[act_ptr_line]->cg();
+    }
+    if(cb!=NULL){
+      *cb=_lines_of_text[act_ptr_line]->cb();
+    }
+    if(dim!=NULL){
+      *dim=_lines_of_text[act_ptr_line]->dim();
+    }
+    if(font!=NULL){
+
+      *font=_lines_of_text[act_ptr_line]->font();
+    }
+    pointer_to_currline(saved_ptr_line);
+    _lines_of_text[pointer_to_currline()]->pointer_to_currline(saved_ptr_line_child);
+    _lines_of_text[act_ptr_line]->treat_as_a_whole(old_t);
+  }
+
 
 }
 
 
+
+
 float paragraph_text::x(){
-  int res=0;
+  float res=0;
 
   if( cairo_t_singleton::can_export()){
     res=phys_x();
@@ -566,7 +646,7 @@ float paragraph_text::x(){
 
 float paragraph_text::visual_x(){
 
-  int res=static_cast<int>(rintf(phys_x()*__pref.getZoom()));
+  float res=(rintf(phys_x()*__pref.getZoom()));
   return res;
 
 }
@@ -578,7 +658,7 @@ float paragraph_text::phys_x(){
 
 
 float paragraph_text::y(){
-  int res=0;
+  float res=0;
 
   if( cairo_t_singleton::can_export()){
     res=phys_y();
@@ -591,7 +671,7 @@ float paragraph_text::y(){
 
 float paragraph_text::visual_y(){
 
-  int res=static_cast<int>(rintf(phys_y()*__pref.getZoom()));
+  float res=phys_y()*__pref.getZoom();
   return res;
 
 }
@@ -742,10 +822,7 @@ float paragraph_text::w(){
     phys_misura(h,w);
   }else{
     visual_misura(h,w);
-
   }
-
-
   return w;
 }
 
@@ -942,6 +1019,7 @@ void paragraph_text::format_paragraph(){
 
   case PARAGRAPH_TEXT_ALIGN_SINGLE_LINE_TEXT_LAYOUT:
     {
+
       for(unsigned int i=0;i<_lines_of_text.size();i++){
         _lines_of_text[i]->y(_y);
         
@@ -970,7 +1048,7 @@ void paragraph_text::draw_cursor(bool draw){
 }
 
 void paragraph_text::disegna(){
-
+			     
   format_paragraph();
 
   if( cairo_t_singleton::can_export()){
@@ -1024,9 +1102,8 @@ void paragraph_text::trasla(float dx, float dy){
 
 
 void paragraph_text::phys_translate(float dx, float dy){
-  x(x()+dx);
-  y(y()+dy);
-  
+  x(phys_x()+dx);
+  y(phys_y()+dy);
 }
 
 
@@ -1052,54 +1129,44 @@ void paragraph_text::ruota(float xpiv, float ypiv,float angl){
 }
 
 
- bool paragraph_text::dentro_bb(float x_bb, float y_bb, float w_bb , float h_bb){
-   //std::cerr << "ooooooo" << std::endl;
-   if(x() > x_bb && x()+w()<x_bb+w_bb &&
-      y() > y_bb && y()+h()<y_bb+h_bb ){
-     return true;
+bool paragraph_text::dentro_bb(float x_bb, float y_bb, float w_bb , float h_bb){
+  std::pair<float, float> ld(x_bb,y_bb);
+  std::pair<float, float> ru(x_bb+w_bb,y_bb+h_bb);
+
+
+  if(can_highlight()){
+    highlight_under_bounding_box(ld,ru);
+  }
+
+  if(x() > x_bb && x()+w()<x_bb+w_bb &&
+     y() > y_bb && y()+h()<y_bb+h_bb ){
+    return true;
   }else{
     return false;
   }
 }
 
-/*
-int paragraph_text::sotto_mouse(int posx_m, int posy_m){
-  float ics_e=x();
-  float ips_e=y();
-  float w_e=w();
-  float h_e=h();
-
-  if(posx_m > ics_e && posx_m < ics_e+w_e &&
-     posy_m > ips_e && posy_m < ips_e+h_e){
-    return 0;
-  }else{
-    return -1;
+void paragraph_text::pointer_to_currline(int nw){
+  if(static_cast<unsigned int>(nw)<_lines_of_text.size()){
+    _pointer_to_currline=nw;
   }
-
 }
-*/
-
- void paragraph_text::pointer_to_currline(int nw){
-   if(static_cast<unsigned int>(nw)<_lines_of_text.size()){
-     _pointer_to_currline=nw;
-   }
- }
  
- int paragraph_text::pointer_to_currline(){
-   return _pointer_to_currline;
- }
+int paragraph_text::pointer_to_currline(){
+  return _pointer_to_currline;
+}
 
 
- int paragraph_text::pointer_to_currline_transl(int delta){
+int paragraph_text::pointer_to_currline_transl(int delta){
   int nw=pointer_to_currline()+delta;
-   int old=pointer_to_currline();
-   if(nw>=0 && static_cast<unsigned int>(nw) < _lines_of_text.size()){
+  int old=pointer_to_currline();
+  if(nw>=0 && static_cast<unsigned int>(nw) < _lines_of_text.size()){
     pointer_to_currline(nw);
-
+    
   }
 
   return old;
-
+  
 }
 
 
@@ -1352,35 +1419,30 @@ bool paragraph_text::cursor_one_step_down(int &curr_type){
   */
 }
 
-
-
 void paragraph_text::insert_string_in_curr_pos(std::string str,int type){
-
   int ptr_line=pointer_to_currline();
-
-  //std::cerr << "ptr_line " << ptr_line << std::endl;
-             
-
   if(ptr_line>=0  && ptr_line_less_lines_text()){
+    bool old_v=_lines_of_text[ptr_line]->treat_as_a_whole(false);
     _lines_of_text[ptr_line]->insert_string_in_curr_pos(str,type);
-    font_child(font());
-    dim_child(phys_dim());
-    color_child(cr(),cg(),cb());
     
+    int phys_dim_l=_lines_of_text[ptr_line]->phys_dim();
+    if(!some_highlighted()){
+      phys_dim_l=phys_dim();
+    }
 
+    _lines_of_text[ptr_line]->treat_as_a_whole(old_v);
+    font_child(font());
+    dim_child(phys_dim_l);
+    color_child(cr(),cg(),cb());
   }else if(_lines_of_text.size()==0){
-    //DEBUG_TO_CERR(phys_dim());
     multifont_label*  tmp=new multifont_label(font(),phys_dim(),cr(),cg(),cb());
     tmp->insert_string_in_curr_pos(str,ET_STR);
     set_cursor_position(0,str.size());
     tmp->reset_cursor_if_outside_limits();
     add_line(tmp);
   }
-
 }
 
-
-
 bool paragraph_text::ptr_line_less_lines_text(){
   bool res=false;
   if(static_cast<unsigned int>(pointer_to_currline()) <  
@@ -1435,10 +1497,11 @@ std::vector<multifont_label*> paragraph_text::get_lines_text(){
 bool paragraph_text::dump_label(){
   std::cerr << __FUNCTION__ << std::endl;
   
-  std::cerr << __FUNCTION__ << x() << std::endl;
-  std::cerr << __FUNCTION__ << y() << std::endl;
-  std::cerr << __FUNCTION__ << interline_space() << std::endl;
-
+  std::cerr << __FUNCTION__ << "x: " << x() << std::endl;
+  std::cerr << __FUNCTION__ << "y: " << y() << std::endl;
+  std::cerr << __FUNCTION__ << "w: " << w() << std::endl;
+  std::cerr << __FUNCTION__ << "h: " << h() << std::endl;
+  std::cerr << __FUNCTION__ << "interline " << interline_space() << std::endl;
   for(unsigned int i =0;i< _lines_of_text.size();i++){
     std::cerr << __FUNCTION__ << _lines_of_text[i]->to_raw_string() 
               << std::endl;
@@ -1490,18 +1553,18 @@ void paragraph_text::format_lines_align_center(){
   float w_max=0;
 
   for(unsigned int i=0;i<_lines_of_text.size();i++){
-    if(_lines_of_text[i]->w() > w_max){
-      w_max=_lines_of_text[i]->w();
+    if(_lines_of_text[i]->phys_w() > w_max){
+      w_max=_lines_of_text[i]->phys_w();
     }
   }
 
   for(unsigned int i=0;i<_lines_of_text.size();i++){
-    _lines_of_text[i]->x(_x + w_max/2 - _lines_of_text[i]->w()/2 );
+    _lines_of_text[i]->x(_x + w_max/2 - _lines_of_text[i]->phys_w()/2 );
 
 
     int total_h=0;
     for(unsigned int j=0;j<i;j++){
-      total_h+=_lines_of_text[j]->h();
+      total_h+=_lines_of_text[j]->phys_h();
       total_h+=phys_interline_space();
 
     }
