@@ -62,7 +62,7 @@
 #include <util.hpp>
 
 
-extern finestra_pr* __la_finestra;
+extern finestra_pr* MainWindow;
 
 extern Preferences  __pref;
 
@@ -570,7 +570,7 @@ void parse_residue::figlio(){
 	assert(hook);
 	for(unsigned int i=0;i<preso;i++){
 	  gruppo branch_added=the_branch;
-	  fondi_gruppi(&_fragments.top(),
+	  merge_groups(&_fragments.top(),
 		       &branch_added,
 		       search_for_attach(&_fragments.top()),
 		       hole, LEGAME_SINGOLO);
