@@ -1751,7 +1751,7 @@ bool  molecule_matching::TEST_DFS(){
   int st=0;
   _sub.DFS_step(&st);
   
-  atomo* res;
+  atomo* res=NULL;
 
 
   _sub.DFS_step();
@@ -1761,7 +1761,7 @@ bool  molecule_matching::TEST_DFS(){
   _sub.DFS_step();
   debug_print_DFS(_sub);
   
-  DEBUG_TO_CERR(_sub.DFS_simulate_shift_descendant(&res));
+  //DEBUG_TO_CERR(_sub.DFS_simulate_shift_descendant(&res));
   if(res){
     std::cerr << "scenderei in " << res->id() << std::endl;
   }
