@@ -28,6 +28,9 @@
 #include <FL/Fl_Pixmap.H>
 #include <global.hpp>
 
+#include <2D_vector.hpp>
+#include <regex.h>
+
 #include <cstdio>
 
 
@@ -57,6 +60,8 @@
 
 #include <prefs.hpp>
 
+#include <font_factory.hpp>
+
 #include <logo.xpm>
 
 #if HAVE_NCURSES_H
@@ -69,9 +74,78 @@ extern Preferences  __pref;
 
 
 
-extern finestra_pr* __la_finestra;
+extern finestra_pr* MainWindow;
 
 
+gruppo create_ring(float side_lenght, float numsides, 
+                   bool aromatic) throw (out_of_range){
+  if(numsides <3){
+    throw out_of_range("numsides < 3");
+
+  }else{
+
+    float alpha=2*M_PI/numsides;
+    float radius=side_lenght/(sqrt(2*(1-cos(alpha))));
+    gruppo nw;
+    
+    for(float ang=0, i=0; i<numsides ; ang+=alpha , i++){
+      atomo tmp;
+      tmp.etich_punt()->aggiungi(NO_ETIC, ET_STR);
+      tmp.pos_x(radius*cos(ang));
+      tmp.pos_y(radius*sin(ang));
+      nw.add_atomo_id(tmp);
+
+    }
+
+    //link atoms
+    vector<atomo>::iterator beg=nw.iniz_atom();
+    vector<atomo>::iterator end=nw.fin_atom();
+
+    int odd_ct=0;
+
+    while(beg < end){
+      int the_id=(*beg).id();;
+    
+      int the_id2=-1;
+
+      if(beg==end-1){
+        the_id2=(*nw.iniz_atom()).id();
+      }else{
+        the_id2=(*(beg+1)).id();
+      }
+
+
+      atomo* a=nw.find_atomo_id(the_id);
+      atomo* b=nw.find_atomo_id(the_id2);
+
+      int type_bd=LEGAME_SINGOLO;
+
+      if(aromatic){
+        if(odd_ct%2==1){
+          type_bd=LEGAME_DOPPIO;
+        }
+
+      }
+
+      a->aggiungi_legame(the_id2,
+                         type_bd,0,
+                         0 ,0);
+      a->costruisci_arrivati();
+      b->aggiungi_legame(the_id,
+                         type_bd,0,
+                         0 ,0);
+      b->costruisci_arrivati();
+      beg++;
+      odd_ct++;
+
+    }
+
+  
+
+    return nw;
+  }
+}
+
 
 
 int remap_color_48(int trentadue){
@@ -174,9 +248,6 @@ void draw_pango_string(std::string st,int x, int y, std::string font,int dim,
                        remap_color_1(cg),
                        remap_color_1(cb));
 
-  //PangoFontDescription* desc=pango_font_description_new();
-  //pango_font_description_set_family(desc,font.c_str());
-
   PangoFontDescription* desc=pango_font_description_from_string(font.c_str());
 
   pango_font_description_set_absolute_size(desc,font_dim_std*PANGO_SCALE);
@@ -204,28 +275,28 @@ float remap_y_ps(float vx){
 bool rett_eqn(float x1, float y1, 
 	      float x2, float y2, 
 	      float& m, float& q){
-  bool risul=true;
+  bool result=true;
   const static float limit_similar=0.1;
-  //if(x1!=x2){ //mah!
+  
   if(!similar_to<float>(x1, x2, limit_similar)){
     m=(y1-y2)/(x1-x2);
     q=y2-m*x2;
   }else{
     m=0;
     q=0;
-    risul=false;
+    result=false;
   }
 
-  return risul;
+  return result;
 
 }
 
 
 
-void trova_perpendicolare(float dax, float day, 
+void is_perpendicular(float dax, float day, 
 			  float ax, float ay, 
 			  bool verso,float gap,
-			  float& risx, float&risy){
+			  float& resx, float &resy){
 
   float m;
   float q;
@@ -235,13 +306,13 @@ void trova_perpendicolare(float dax, float day,
   float treposx=ax-dax; 
   float treposy=ay-day;
   
-  bool risultato=rett_eqn(trposx,trposy,
+  bool result=rett_eqn(trposx,trposy,
 			  treposx,treposy,
 			  m,q);
   
   float alfafr;
   
-  if(risultato){
+  if(result){
     alfafr=atan(-1/m);
   }else{
     alfafr=0;
@@ -251,15 +322,15 @@ void trova_perpendicolare(float dax, float day,
     gap=-gap;
   }
   
-  risx=gap*cos(alfafr);
-  risy=gap*sin(alfafr);
+  resx=gap*cos(alfafr);
+  resy=gap*sin(alfafr);
 
   
 }
 
 
 
-void disegna_parallela(float dax, float day, 
+void draw_parallel(float dax, float day, 
                        float ax, float ay,
                        int cr, int cg, int cb,
                        bool verso,float gap,bool tratt, bool noaccorc){
@@ -296,18 +367,11 @@ void disegna_parallela(float dax, float day,
     alfafr=0;
   }
 
-  /*
-    fl_pie(static_cast<int>(lung*cos(alfafr)+dax+scompx),
-	   static_cast<int>(lung*sin(alfafr)+day+scompy ), 3, 3, 0, 360.0);
-  */
-
-
-    bool semip= ritorna_semipiano_app(lung*cos(alfafr)+dax+scompx, 
+    bool semip= return_half_plane(lung*cos(alfafr)+dax+scompx, 
 				      lung*sin(alfafr)+day+scompy, 
 				      dax, day,
 				      ax, ay );
 
-    //    cout << semip << " " << verso << endl;
 
     if(!(semip==verso)){
 	lung=-lung;
@@ -357,18 +421,13 @@ void disegna_parallela(float dax, float day,
     }
 }
 
-void disegna_freccia(float dax, float day, 
+void draw_arrow(float dax, float day, 
 		     float ax, float ay, 
-		     float spessore,float arr_w, 
+		     float thickness,float arr_w, 
 		     float arr_h, float arr_gap,
                      int cr, int cg, int cb,
 		     bool verso){
 
-  
-
-
-
-
   float the_zoom=__pref.getZoom();
   float m;
   float q;
@@ -387,19 +446,14 @@ void disegna_freccia(float dax, float day,
     
     float ecompy_fin=(arr_w-arr_gap*arr_w)*sin(alfa);
     float ecompx_fin=(arr_w-arr_gap*arr_w)*cos(alfa);
-    
-    /*
-    float ecompy_fin=(arr_w)*sin(alfa);
-    float ecompx_fin=(arr_w)*cos(alfa);   
-    */
     float alfafr=atan(-1/m);
 
     if(m==0){ //mah?
       alfafr=M_PI/2;
     }
 
-    float lung=arr_h+spessore;
-    float lungspess=static_cast<float>(spessore)/2.;
+    float lung=arr_h+thickness;
+    float lungspess=static_cast<float>(thickness)/2.;
     	
     if(verso){
       lung=-lung;
@@ -418,7 +472,7 @@ void disegna_freccia(float dax, float day,
 	  
     float risx=0, risy=0;
 	
-    trova_intersezione(lato1,lato2, risx, risy);
+    is_intersection(lato1,lato2, risx, risy);
 
     if(cairo_t_singleton::can_export()){
       
@@ -446,9 +500,9 @@ void disegna_freccia(float dax, float day,
     
     }else{
 
-      fl_line_style(FL_SOLID,static_cast<int>(rintf(spessore)));
-      int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-      int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
+      fl_line_style(FL_SOLID,static_cast<int>(rintf(thickness)));
+      int dx_scroll=(MainWindow->ritorna_mol_canvas())->x();
+      int dy_scroll=(MainWindow->ritorna_mol_canvas())->y();
 
       float dx_sc=risx;//lu.first+w_sc/2;
       float dy_sc=risy;//lu.second+h_sc/2;
@@ -479,24 +533,17 @@ void disegna_freccia(float dax, float day,
   }else{
 
     float spy=arr_w; //displ on y
-    float spx=arr_h+spessore; //displ on x
-    float lungspess=spessore/2.;
+    float spx=arr_h+thickness; //displ on x
+    float lungspess=thickness/2.;
 	
     if(verso){// revert displ 
       spx=-spx;  
       lungspess=-lungspess;
-      //std::cerr << " spx " << spx << " " << lungspess << std::endl;
+      
     }
-    
-
 
     if(day>ay){
       spy=-spy;
-      // std::cerr << "day " << day << " " << ay << std::endl;
-//       std::cerr << " spx " << spx << " spessore " <<  spessore << " lungspess "<< lungspess << std::endl;
-//       std::cerr << " spy " << spy << std::endl;
-    }else{
-
     }
     
     float lato1[4]={
@@ -508,23 +555,8 @@ void disegna_freccia(float dax, float day,
       ax,ay
     };
 	  
-    /*
-
-    for(unsigned int i =0;i<4;i++){
-      cerr << lato1[i] << std::endl;
-    }
-    
-    for(unsigned int i =0;i<4;i++){
-      cerr << lato2[i] << std::endl;
-    }
-
-    std::cout << "-------------------" << std::endl;  
-    */
-
-
-
     float risx=0, risy=0;
-    trova_intersezione(lato1,lato2, risx, risy);
+    is_intersection(lato1,lato2, risx, risy);
 
     if(cairo_t_singleton::get_context()){
       cairo_t* cn=cairo_t_singleton::get_context();
@@ -544,9 +576,9 @@ void disegna_freccia(float dax, float day,
 
     }else{
 
-      fl_line_style(FL_SOLID,static_cast<int>(rintf(spessore)));
-      int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-      int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
+      fl_line_style(FL_SOLID,static_cast<int>(rintf(thickness)));
+      int dx_scroll=(MainWindow->ritorna_mol_canvas())->x();
+      int dy_scroll=(MainWindow->ritorna_mol_canvas())->y();
 
       float dx_sc=risx;//lu.first+w_sc/2.0;
       float dy_sc=risy;//lu.second+h_sc/2.0;
@@ -574,12 +606,12 @@ void disegna_freccia(float dax, float day,
 
 void calculate_arrow_points(float dax, float day, 
                             float ax, float ay, 
-                            float spessore,float arr_w, 
+                            float thick,float arr_w, 
                             float arr_h, float arr_gap,
                             int cr, int cg, int cb,
                             std::vector<float>& results_x,
                             std::vector<float>& results_y,
-                            bool verso){
+                            bool to){
 
   float the_zoom=__pref.getZoom();
   float m;
@@ -589,10 +621,10 @@ void calculate_arrow_points(float dax, float day,
   float treposx=dax-ax; 
   float treposy=day-ay;
 
-  bool risultato=rett_eqn(trposx,trposy,
+  bool result=rett_eqn(trposx,trposy,
 			  treposx,treposy,
 			  m,q);
-  if(risultato){
+  if(result){
 
     float alfa=atan2(treposy,treposx);
     float ecompy=arr_w*sin(alfa);
@@ -600,21 +632,16 @@ void calculate_arrow_points(float dax, float day,
     
     float ecompy_fin=(arr_w-arr_gap*arr_w)*sin(alfa);
     float ecompx_fin=(arr_w-arr_gap*arr_w)*cos(alfa);
-    
-    /*
-    float ecompy_fin=(arr_w)*sin(alfa);
-    float ecompx_fin=(arr_w)*cos(alfa);   
-    */
     float alfafr=atan(-1/m);
 
     if(m==0){ //mah?
       alfafr=M_PI/2;
     }
 
-    float lung=arr_h+spessore;
-    float lungspess=static_cast<float>(spessore)/2.;
+    float lung=arr_h+thick;
+    float lungspess=static_cast<float>(thick)/2.;
     	
-    if(verso){
+    if(to){
       lung=-lung;
       lungspess=-lungspess;
     }
@@ -629,9 +656,9 @@ void calculate_arrow_points(float dax, float day,
       ax,ay
     };
 	  
-    float risx=0, risy=0;
+    float resx=0, resy=0;
 	
-    trova_intersezione(lato1,lato2, risx, risy);
+    is_intersection(lato1,lato2, resx, resy);
 
 
 
@@ -642,8 +669,8 @@ void calculate_arrow_points(float dax, float day,
       float back_point_y_w_gap=ecompy_fin+ay;
       
 
-      results_x.push_back(risx);
-      results_y.push_back(risy);
+      results_x.push_back(resx);
+      results_y.push_back(resy);
 
       results_x.push_back(back_point_x);
       results_y.push_back(back_point_y);
@@ -653,15 +680,11 @@ void calculate_arrow_points(float dax, float day,
 
     }else{
 
-      
-      // int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-      // int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
+      float dx_sc=resx;//lu.first+w_sc/2;
+      float dy_sc=resy;//lu.second+h_sc/2;
 
-      float dx_sc=risx;//lu.first+w_sc/2;
-      float dy_sc=risy;//lu.second+h_sc/2;
-
-      float p1x=scale_center_point(risx,dx_sc, the_zoom);
-      float p1y=scale_center_point(risy,dy_sc, the_zoom);
+      float p1x=scale_center_point(resx,dx_sc, the_zoom);
+      float p1y=scale_center_point(resy,dy_sc, the_zoom);
 
       float p2x=scale_center_point(lung*cos(alfafr)+ax+ecompx,dx_sc, the_zoom);
       float p2y=scale_center_point(lung*sin(alfafr)+ay+ecompy,dy_sc, the_zoom);
@@ -686,24 +709,18 @@ void calculate_arrow_points(float dax, float day,
   }else{
 
     float spy=arr_w; //displ on y
-    float spx=arr_h+spessore; //displ on x
-    float lungspess=spessore/2.;
+    float spx=arr_h+thick; //displ on x
+    float lungspess=thick/2.;
 	
-    if(verso){// revert displ 
+    if(to){// revert displ 
       spx=-spx;  
       lungspess=-lungspess;
-      //std::cerr << " spx " << spx << " " << lungspess << std::endl;
     }
     
 
 
     if(day>ay){
       spy=-spy;
-      // std::cerr << "day " << day << " " << ay << std::endl;
-//       std::cerr << " spx " << spx << " spessore " <<  spessore << " lungspess "<< lungspess << std::endl;
-//       std::cerr << " spy " << spy << std::endl;
-    }else{
-
     }
     
     float lato1[4]={
@@ -715,28 +732,13 @@ void calculate_arrow_points(float dax, float day,
       ax,ay
     };
 	  
-    /*
-
-    for(unsigned int i =0;i<4;i++){
-      cerr << lato1[i] << std::endl;
-    }
-    
-    for(unsigned int i =0;i<4;i++){
-      cerr << lato2[i] << std::endl;
-    }
-
-    std::cout << "-------------------" << std::endl;  
-    */
-
-
-
-    float risx=0, risy=0;
-    trova_intersezione(lato1,lato2, risx, risy);
+    float resx=0, resy=0;
+    is_intersection(lato1,lato2, resx, resy);
 
     if(cairo_t_singleton::get_context()){
 
-      results_x.push_back(risx);
-      results_y.push_back(risy);
+      results_x.push_back(resx);
+      results_y.push_back(resy);
 
 
       results_x.push_back(ax+spx);
@@ -748,14 +750,11 @@ void calculate_arrow_points(float dax, float day,
 
     }else{
 
-      // int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-      // int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
-
-      float dx_sc=risx;//lu.first+w_sc/2.0;
-      float dy_sc=risy;//lu.second+h_sc/2.0;
+      float dx_sc=resx;//lu.first+w_sc/2.0;
+      float dy_sc=resy;//lu.second+h_sc/2.0;
       
-      float p1x=scale_center_point(risx,dx_sc, the_zoom);
-      float p1y=scale_center_point(risy,dy_sc, the_zoom);
+      float p1x=scale_center_point(resx,dx_sc, the_zoom);
+      float p1y=scale_center_point(resy,dy_sc, the_zoom);
 
       float p2x=scale_center_point(ax+spx,dx_sc, the_zoom);
       float p2y=scale_center_point(ay-spy,dy_sc, the_zoom);
@@ -783,91 +782,81 @@ void calculate_arrow_points(float dax, float day,
 
 
 
-bool ritorna_semipiano_app(float x, float y, 
+bool return_half_plane(float x, float y, 
 			   float xrt, float yrt,
 			   float xrt2, float yrt2 ){
 
   float m;
   float q;
-  //cout << "xrt" << xrt << " yrt " << yrt << "xrt2 " << xrt2 << " yrt2 " << yrt2 << endl;
   bool rest=rett_eqn(xrt,yrt,xrt2,yrt2,m,q);
-  bool risul;
+  bool res;
   if(rest){
-    risul = y >= m*x + q;
+    res = y >= m*x + q;
   }else{
-    risul = x > xrt;
+    res = x > xrt;
   }
-  //cout << "y " << y << "\tm " << m << "\tx " << x << "\tq " << q << "rest" << rest << "\t" <<risul << endl;
-
-  return risul;
+  return res;
 
 }
 
 
 
-float b_clip_v(float da_x, float da_y, float fino_x, float fino_y, float x_edge){
-  //  float y_ris;
-  float xpiv=min(da_x,fino_x);
-  float x1=min(da_x,fino_x);
-  float x2=max(da_x,fino_x);
-  float y1= x1==da_x ? da_y : fino_y;
-  float y2= x2==da_x ? da_y : fino_y;
+float b_clip_v(float da_x, float da_y, float end_x, float end_y, float x_edge){
+  float xpiv=min(da_x,end_x);
+  float x1=min(da_x,end_x);
+  float x2=max(da_x,end_x);
+  float y1= x1==da_x ? da_y : end_y;
+  float y2= x2==da_x ? da_y : end_y;
 
   float m ,q;
 
   bool risul=rett_eqn(x1,y1, 
 		      x2, y2, 
 		      m, q); 
-  //  cout << x1 << " " << x2 << " " << (x2+x1)/2 <<endl;  
   if(risul){
 
     while(abs(xpiv-x_edge)>0.001){
       xpiv=(x2+x1)/2;
-      //cout <<  x1 << "x2 " << x2 << "xpiv " << xpiv << "xedge " 
-      //	   << x_edge << " " <<abs(xpiv-x_edge) << " "<< (abs(xpiv-x_edge)>0.001) <<  endl;
-      //cout <<  x1 << " " << x2 << " " << xpiv << " " << x_edge << endl;
-      //sleep(2);
+
       if(xpiv<x_edge){
 	x1=xpiv;
       }else{
 	x2=xpiv;
       }
     }
-    //cout << "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii" << endl;
+
     return xpiv*m+q;
 
   }else{
     //non  dovrebbe arrivarci  mai visto  che il  segmento non  sara mai
     //parallelo alla y
-    cout << "Come e' che sono qui?" << endl;
+    cout << "b_clip_v: unexpected value returned by \'rett_eqn\' function\n" << endl;
     return -1;
   }
 
-
 }
 
 
-float b_clip_h(float da_x, float da_y, float fino_x, float fino_y, float y_edge){
+float b_clip_h(float da_x, float da_y, float end_x, float end_y, float y_edge){
 
-  float ypiv=min(da_y,fino_y);
-  float y1=min(da_y,fino_y);
-  float y2=max(da_y,fino_y);
+  float ypiv=min(da_y,end_y);
+  float y1=min(da_y,end_y);
+  float y2=max(da_y,end_y);
 
-  float x1= y1==da_y ? da_x : fino_x;
-  float x2= y2==da_y ? da_x : fino_x;
+  float x1= y1==da_y ? da_x : end_x;
+  float x2= y2==da_y ? da_x : end_x;
 
   float m ,q;
 
   bool risul=rett_eqn(x1,y1, 
 		      x2, y2, 
 		      m, q); 
-  //  cout << x1 << " " << x2 << " " << (x2+x1)/2 <<endl;  
+
   if(risul){
 
     while(abs(ypiv-y_edge)>0.001){
       ypiv=(y2+y1)/2;
-      //cout <<  y1 << " " << y2 << " " << ypiv << " " << y_edge << endl;
-      //sleep(2);
+
       if(ypiv<y_edge){
 	y1=ypiv;
       }else{
@@ -883,28 +872,26 @@ float b_clip_h(float da_x, float da_y, float fino_x, float fino_y, float y_edge)
 
 
 
-bool appartiene_retta(int x_m, int y_m, 
+bool line_belongs(int x_m, int y_m, 
 		     float st_x, float st_y,
 		     float e_x, float e_y,
-		     int spessore){
-
-
+		     int thickness){
 
   float m;
   float q;
 
-  if(abs(st_x-e_x)<spessore){
+  if(abs(st_x-e_x) < thickness){
     e_x=st_x;
   }
 
-  bool risultato=rett_eqn(st_x,st_y,
+  bool result=rett_eqn(st_x,st_y,
 			  e_x,e_y,
 			  m,q);
-  //cout << " " << st_x << " " << st_y << endl;
-  if(risultato){
+
+  if(result){
 
     float yprev=m * x_m + q;
-    bool similar=similar_to(yprev, static_cast<float>(y_m), static_cast<float>(spessore));
+    bool similar=similar_to(yprev, static_cast<float>(y_m), static_cast<float>(thickness));
 
     if(similar){
       if(e_x - st_x >0){
@@ -916,24 +903,8 @@ bool appartiene_retta(int x_m, int y_m,
       return false;
     }
 
-    //     int yprev=static_cast<int>(rintf( m * x_m + q));
-    //     /*    
-    //     cout << "appartiene retta m " << m << "q " << q << "x_m " << x_m << "y_m "
-    //     << y_m << "y_prev " << yprev << "--- " << abs(static_cast<int>(yprev-y_m)) << endl;
-    //     */
-    //     if( abs(static_cast<int>(yprev-y_m))<spessore &&
-    // 	x_m<max(st_x,e_x) &&
-    // 	x_m>min(st_x,e_x)){
-    
-    //       return true;
-    //     }else{
-    //       return false;
-    //     }
-
-    
-
   }else{
-    if(abs(x_m - static_cast<int>(rintf(st_x))) < spessore &&
+    if(abs(x_m - static_cast<int>(rintf(st_x))) < thickness &&
        y_m<max(st_y,e_y) &&
        y_m>min(st_y,e_y)){
       return true;
@@ -945,7 +916,7 @@ bool appartiene_retta(int x_m, int y_m,
 }
 
 
-void disegna_pallino(float x, float y){
+void draw_point(float x, float y){
   int raggioatt=static_cast<int>(rintf(RAGGIO_EL*1.5));
   int xint=static_cast<int>(rintf(x));
   int yint=static_cast<int>(rintf(y));
@@ -968,12 +939,12 @@ void draw_shadowed_point(float x, float y, int r, int g, int b){
 }
 
 
-void disegna_cornice_pallini(int x, int y , int w ,int h){
+void draw_frame(int x, int y , int w ,int h){
 
-  disegna_pallino(x,y);
-  disegna_pallino(x+w,y);
-  disegna_pallino(x+w,y+h);
-  disegna_pallino(x,y+h);
+  draw_point(x,y);
+  draw_point(x+w,y);
+  draw_point(x+w,y+h);
+  draw_point(x,y+h);
 }
 
 
@@ -1071,68 +1042,6 @@ void get_bb_etic(etichetta& et, int& x, int& y){
 
 }
 
-/*
-void get_bb_etic(etichetta& et, float& x, float& y){
-
-  string font_ps;
-  int tipo_font;
-
-
-  get_font_handle(et, tipo_font, font_ps);
-
-  fl_font(tipo_font,et.phys_dim());
-
-
-
-  float st_xpos=et.phys_x();
-  float st_ypos=et.phys_y();
-
-
-  int half_w=0;
-  int h=0;
-  string car;
-  vector < pair<string,int> > stringhevec=et.vec_str();
-  
-
-  switch(et.allineamento()){
-
-  case ETIC_ALLINEA_CT:
-    st_xpos-=et.phys_w()/2;
-    st_ypos-=et.phys_h()/2;
-    break;
-  case ETIC_ALLINEA_DX:
-    if(stringhevec.size()>0){
-      car=stringhevec[(stringhevec.size()-1)].first;
-    }
-    
-
-    if(car.size()>0){
-      car=car[car.size()-1];
-      fl_measure(car.c_str(), half_w, h);
-    }
-
-
-    st_xpos-=et.phys_w();
-    st_xpos+=half_w/2;
-    st_ypos-=et.phys_h()/2;
-    break;
-  case ETIC_ALLINEA_SX:
-  default:
-    car=((et.vec_str())[0].first)[0];
-    fl_measure(car.c_str(), half_w, h);
-    //cout << "iioval " << car << " " << half_w << " " << et.h() << endl;
-    st_ypos-=et.phys_h()/2;
-    st_xpos-=half_w/2;
-    break;
-  }
-
-  x=static_cast<int>(rintf(st_xpos));
-  y=static_cast<int>(rintf(st_ypos+et.v_offset()));
-
-}
-
-*/
-
 void get_bb_etic(etichetta& et, float& x, float& y){
   string font_ps;
   int tipo_font;
@@ -1146,10 +1055,6 @@ void get_bb_etic(etichetta& et, float& x, float& y){
 
   float st_xpos=et.phys_x();
   float st_ypos=et.phys_y();
-
-
-  //int half_w=0;
-  //int h=0;
   string car;
   vector < pair<string,int> > stringhevec=et.vec_str();
   
@@ -1198,77 +1103,80 @@ void get_bb_etic(etichetta& et, float& x, float& y){
 
 void get_font_handle(etichetta et, int &font_fltk, string &font_libplot){
 
+  font_factory* f_factory=font_map_singleton::get_context();
+
+
   string font_ps;
   int tipo_font;
   switch(et.font()){
 
   case 0:
-    tipo_font=FL_HELVETICA;
+    tipo_font=f_factory->get_actual_font(FL_HELVETICA);
     font_ps=__pref.get_sans_font().c_str();
     break;
   case 1:
-    tipo_font=FL_HELVETICA_BOLD;
+    tipo_font=f_factory->get_actual_font(FL_HELVETICA_BOLD);
     font_ps=__pref.get_sans_font_bold().c_str();
     break;
   case 2:
-    tipo_font=FL_HELVETICA_ITALIC;
+    tipo_font=f_factory->get_actual_font(FL_HELVETICA_ITALIC);
     font_ps=__pref.get_sans_font_italic().c_str();
     break;
   case 3:
-    tipo_font=FL_HELVETICA_BOLD_ITALIC;
+    tipo_font=f_factory->get_actual_font(FL_HELVETICA_BOLD_ITALIC);
     font_ps=__pref.get_sans_font_bold_italic().c_str();
     break;
   case 4:
-    tipo_font=FL_COURIER;
+    tipo_font=f_factory->get_actual_font(FL_COURIER);
     font_ps=__pref.get_mono_font().c_str();
     break;
-  case 5:
-    tipo_font=FL_COURIER_BOLD;
+    case 5:
+      tipo_font=f_factory->get_actual_font(FL_COURIER_BOLD);
     font_ps=__pref.get_mono_font_bold().c_str();
     break;
   case 6:
-    tipo_font=FL_COURIER_ITALIC;
+    tipo_font=f_factory->get_actual_font(FL_COURIER_ITALIC);
     font_ps=__pref.get_mono_font_italic().c_str();
     break;
   case 7:
-    tipo_font=FL_COURIER_BOLD_ITALIC;
+    tipo_font=f_factory->get_actual_font(FL_COURIER_BOLD_ITALIC);
     font_ps=__pref.get_mono_font_bold_italic().c_str();
     break;
   case 8:
-    tipo_font=FL_TIMES;
+    tipo_font=f_factory->get_actual_font(FL_TIMES);
     font_ps=__pref.get_serif_font().c_str();
     break;
   case 9:
-    tipo_font=FL_TIMES_BOLD;
+    tipo_font=f_factory->get_actual_font(FL_TIMES_BOLD);
     font_ps=__pref.get_serif_font_bold().c_str();
     break;
   case 10:
-    tipo_font=FL_TIMES_ITALIC;
+    tipo_font=f_factory->get_actual_font(FL_TIMES_ITALIC);
     font_ps=__pref.get_serif_font_italic().c_str();
     break;
   case 11:
-    tipo_font=FL_TIMES_BOLD_ITALIC;
+    tipo_font=f_factory->get_actual_font(FL_TIMES_BOLD_ITALIC);
     font_ps=__pref.get_serif_font_bold_italic().c_str();
     break;
   case 12:
-    tipo_font=FL_SYMBOL;
+    tipo_font=f_factory->get_actual_font(FL_SYMBOL);
     font_ps=__pref.get_symbol_font().c_str();
     break;
   case 13:
-    tipo_font=FL_SCREEN;
+    tipo_font=f_factory->get_actual_font(FL_SCREEN);
     font_ps=__pref.get_mono_font().c_str();
     break;
   case 14:
-    tipo_font=FL_SCREEN_BOLD;
+    tipo_font=f_factory->get_actual_font(FL_SCREEN_BOLD);
     font_ps=__pref.get_mono_font_bold().c_str();
     break;
   case 15:
-    tipo_font=FL_ZAPF_DINGBATS;
+    tipo_font=f_factory->get_actual_font(FL_ZAPF_DINGBATS);
     font_ps=__pref.get_dingbats_font().c_str();
   break;
 
  default:
-   tipo_font=FL_TIMES;
+   tipo_font=f_factory->get_actual_font(FL_TIMES);
    font_ps=__pref.get_serif_font().c_str();
    break;
 
@@ -1277,7 +1185,6 @@ void get_font_handle(etichetta et, int &font_fltk, string &font_libplot){
   font_fltk=tipo_font;
   font_libplot=font_ps;
 
-
 }
 
 
@@ -1318,7 +1225,7 @@ unsigned int create_hash(string value){
 
 
 
-bool trova_intersezione(float* xy1,float* xy2, float& risx, float& risy){
+bool is_intersection(float* xy1,float* xy2, float& risx, float& risy){
   float m1=-220, q1=-20;
   bool prima_retta=rett_eqn(xy1[0], xy1[1], 
 			    xy1[2], xy1[3], 
@@ -1351,17 +1258,23 @@ bool trova_intersezione(float* xy1,float* xy2, float& risx, float& risy){
 
 
 
-splash_window::splash_window(int w, int h)
-  :Fl_Window(w,h)
+Splash_Window::Splash_Window()
+  :Fl_Window(Splash_Window::splash_w,Splash_Window::splash_h),
+   _logo(0,0,Splash_Window::splash_w,Splash_Window::splash_h)
 {
-  
+  clear_border();
+  resize(Fl::w()/2 - w()/2,Fl::h()/2 - h()/2,Splash_Window::splash_w,Splash_Window::splash_h);
+  box(FL_FLAT_BOX);
+  color(FL_BLACK);
+  add(_logo);
+  Fl::add_timeout(Splash_Window::splash_persistence, Splash_Window::hide_win_timeout, reinterpret_cast<void*>(this));
 }
   
-splash_window::~splash_window() { 
-
+Splash_Window::~Splash_Window() { 
+  Fl::remove_timeout(Splash_Window::hide_win_timeout, reinterpret_cast<void*>(this));
 }
   
-int splash_window::handle(int e){
+int Splash_Window::handle(int e){
   switch(e){
   case FL_ENTER:
     hide();
@@ -1371,26 +1284,36 @@ int splash_window::handle(int e){
 }
 
 
-splash_widget::splash_widget(int x,int y,int w, int h)
+
+void Splash_Window::hide_win_timeout(void* win){
+  Fl_Window* w=reinterpret_cast<Fl_Window*>(win);
+  w->hide();
+}
+
+
+const double Splash_Window::splash_persistence=5;
+
+const int Splash_Window::splash_w=452;
+const int Splash_Window::splash_h=348;
+
+Splash_Widget::Splash_Widget(int x,int y,int w, int h)
   :Fl_Widget(x, y, w, h)
 {
 
 }
 
-splash_widget::~splash_widget(){
+Splash_Widget::~Splash_Widget(){
 
 }
 
-void splash_widget::draw(){
+void Splash_Widget::draw(){
   fl_draw_pixmap(logo, 2, 2);
 }
 
 
 
-void fondi_gruppi(gruppo* g_st , gruppo* g_arr,atomo* st, atomo* arr, int tipo){
+void merge_groups(gruppo* g_st , gruppo* g_arr,atomo* st, atomo* arr, int tipo){
 
-  //cout << "cerco di legare " << st->id() << " " << arr->id() << endl;
-  //cout << "crea_legame_nuovo FUNZICA" << endl;
   //fare il join di due gruppi diversi
   int id_atom_arr_bk=arr->id();
 
@@ -1398,7 +1321,7 @@ void fondi_gruppi(gruppo* g_st , gruppo* g_arr,atomo* st, atomo* arr, int tipo){
 
   int id_nuovo_arr=arr->id();;
   
-  //id del piu' grande degli id del gruppo cui st appartiene
+  //the largest id in the st group
   int id_st_max=(*(g_st->fin_atom()-1)).id();
   
   //cerchiamo se nel gruppo di arr esiste id_st_max
@@ -1447,7 +1370,7 @@ void fondi_gruppi(gruppo* g_st , gruppo* g_arr,atomo* st, atomo* arr, int tipo){
   atomo* nuovoarr=0;
   atomo* nuovost=0;
   while(inatm4!=finatm4){
-    //cout << "oo" << (*inatm4).id() << endl;
+
     if((*inatm4).id()==id_legato){
       nuovoarr=&(*inatm4);
     }
@@ -1458,13 +1381,6 @@ void fondi_gruppi(gruppo* g_st , gruppo* g_arr,atomo* st, atomo* arr, int tipo){
     inatm4++;
   }
 
-    //cout << "iiiii " << nuovoarr  << endl;
-
-    //cout << nuovost->id_gruppo() << "==" << nuovoarr->id_gruppo() << endl;
-
-
-
-
   nuovost->aggiungi_legame(nuovoarr->id(),tipo,0,0,0);
   nuovost->costruisci_arrivati();
   
@@ -1489,6 +1405,34 @@ void fondi_gruppi(gruppo* g_st , gruppo* g_arr,atomo* st, atomo* arr, int tipo){
 
 }
 
+bool calc_bb_gen_intersect(pair<float,float>& lu1,
+                           pair<float,float>& rd1,
+                           pair<float,float>& lu2,
+                           pair<float,float>& rd2
+                           ){
+  
+  bool res=true;
+
+  
+  if(lu1.first >  rd2.first) { //OK
+    res=false;
+  }
+
+  if(rd1.first <  lu2.first ){
+    res=false;
+  }
+
+  if(lu1.second >  rd2.second ){
+    res=false;
+  }
+
+
+  if(rd1.second <  lu2.second ){ //OK
+    res=false;
+  } 
+  return res;
+}
+
 void calc_bb_gen(vector<float> x, vector<float> y, 
 		 pair<float,float>& ld,
 		 pair<float,float>& ru){
@@ -1496,6 +1440,9 @@ void calc_bb_gen(vector<float> x, vector<float> y,
   float min_ybb=1E20;
   float max_xbb=1E-20;
   float max_ybb=1E-20;
+
+
+
   for(unsigned int i=0;i<x.size();i++){
     if(x[i]< min_xbb){
       min_xbb=x[i];
@@ -1545,9 +1492,9 @@ vector<float> correct_arrow(float xtan_st, float ytan_st,
   float pt_y=ytan_end;
   float x_c_diff=xtan_end-xtan_st;
   float y_c_diff=ytan_end-ytan_st;
-  //modulo
+  //module
   float mgn_diff=sqrt(pow(x_c_diff,2)+pow(y_c_diff,2));
-  //angolo
+  //angle
   float angl_diff=atan2f(y_c_diff,x_c_diff);
   if(mgn_diff<20){
     pt_x=xtan_st+( (mgn_diff*5/8) * cos(angl_diff));
@@ -1564,15 +1511,15 @@ vector<float> correct_arrow(float xtan_st, float ytan_st,
 
 int ask_overwrite_file(string the_file){
   string the_choice=string(_("File with name")) + 
-    string(_(" ")) + string(the_file) + string(_(" ")) +
+    string(" ") + string(the_file) + string(" ") +
     string(_("already exist.\nDo you want to overwrite it?"));
-  return fl_choice(the_choice.c_str(),_("No"),_("Yes"),NULL);
+  return fl_choice("%s",_("No"),_("Yes"),NULL,the_choice.c_str());
     
 }
 
 int ask_overwrite_file_console(string the_file){
   string the_choice=string(_("File with name")) + 
-    string(_(" ")) + string(the_file) + string(_(" ")) +
+    string(" ") + string(the_file) + string(" ") +
     string(_("already exist.\nDo you want to overwrite it?[y/N] "));
 
   std::string ans;
@@ -1651,12 +1598,13 @@ void print_redfog_wisdom(){
     "...\"Caso mai passo\"...                  ", //10
     "...\"Hai martello e scarpe da tennis?\"...",//11
     "...\"Hai mollica?\"...                    ",//12
-    "...\"Qualcuno sa perche' l'istallazione di Debian sull'amiga 2000 mi si fermi al 2%\"...",//13
+    "...\"Qualcuno sa perche' l'installazione di Debian sull'amiga 2000 mi si fermi al 2%\"...",//13
     "...\"Motoguzzi\"...                                                                    ",//14
-    "\"ma sei pazzo?\nDove cazzo sei?\nMi risponde una voce in inglese!\"\n-- sms spedito a cage mentre questi e' in vacanza"//15
+    "\"ma sei pazzo?\nDove cazzo sei?\nMi risponde una voce in inglese!\"\n-- sms spedito a cage mentre questi e' in vacanza",//15
+    "...\"nelle moderne CPU il bus della cialda di silicio e' ormai cosi'\npiccolo da essere diventato puntiforme\"..."//16
   };
   const static int start_wisdom=10;
-  const static int end_wisdom=15;
+  const static int end_wisdom=16;
   srand(time(NULL));
 
   unsigned int perl=( rand()%(end_wisdom - start_wisdom + 1 ) ) + start_wisdom;
@@ -1715,32 +1663,36 @@ float deg2rad(float angle){
 
 
 void attract_mouse_to_magnetic_point(int* x_m, int* y_m){
-  float mag_gap=__pref.getMagnetic_point_gap();
-  if(mag_gap>0){
-    int dx_scroll=(__la_finestra->ritorna_mol_canvas())->x();
-    int dy_scroll=(__la_finestra->ritorna_mol_canvas())->y();
-    
-    int mult_x=static_cast<int>(static_cast<float>(((*x_m)-dx_scroll))/mag_gap);
-  
-    int mult_y=static_cast<int>(static_cast<float>(((*y_m)-dy_scroll))/mag_gap);
-    
-    //similar_to(T v, T ref, T offset)
-    
-    int limit_x=mult_x*static_cast<int>(mag_gap);
-    int limit_y=mult_y*static_cast<int>(mag_gap);
 
-    if(similar_to(limit_x,(*x_m-dx_scroll),static_cast<int>(mag_gap/2))){
-      (*x_m)=limit_x + dx_scroll;
-    }else{
-      (*x_m)=limit_x + dx_scroll + mag_gap;
-    }
+  double mag_gap=__pref.getMagnetic_point_gap() * __pref.getZoom();
+  if(mag_gap>0){
+    int dx_scroll=(MainWindow->ritorna_mol_canvas())->x();
+    int dy_scroll=(MainWindow->ritorna_mol_canvas())->y();
+    *x_m-=dx_scroll;
+    *y_m-=dy_scroll;
     
-    if(similar_to(limit_y,(*y_m-dx_scroll),static_cast<int>(mag_gap/2))){
-      (*y_m)=limit_y + dy_scroll;
-    }else{
-      (*y_m)=limit_y + dy_scroll + mag_gap;
-    }
+    int floor_x=static_cast<int>(floor((*x_m)/mag_gap));
+    int floor_y=static_cast<int>(floor((*y_m)/mag_gap));
     
+    int ceil_x=static_cast<int>(ceil((*x_m)/mag_gap));
+    int ceil_y=static_cast<int>(ceil((*y_m)/mag_gap));
+    if(abs(*x_m - (ceil_x * mag_gap)) <
+       abs(*x_m - (floor_x * mag_gap))){
+      (*x_m)=static_cast<int>(rintf(ceil_x * mag_gap));
+    }else{
+      (*x_m)=static_cast<int>(rintf(floor_x* mag_gap));
+    }
+
+    *x_m+=dx_scroll;
+
+    if(abs(*y_m - (ceil_y * mag_gap)) <
+       abs(*y_m - (floor_y * mag_gap))){
+      (*y_m)=static_cast<int>(rintf(ceil_y* mag_gap));
+    }else{
+      (*y_m)=static_cast<int>(rintf(floor_y* mag_gap));
+    }
+    *y_m+=dy_scroll;
+
   }
 }
 
@@ -1766,9 +1718,6 @@ void search_for_font(const char* desidered, Fl_Font destination, int num_fonts){
 void set_actual_font_from_prefs(){
   int n=Fl::set_fonts(NULL);
   search_for_font(__pref.get_sans_font().c_str(),FL_HELVETICA,n);
-
-
-
   search_for_font(__pref.get_sans_font().c_str(),FL_HELVETICA,n);
   search_for_font(__pref.get_sans_font_bold().c_str(),FL_HELVETICA_BOLD,n);
   search_for_font(__pref.get_sans_font_italic().c_str(),FL_HELVETICA_ITALIC,n);
@@ -1791,28 +1740,6 @@ void set_actual_font_from_prefs(){
 
 bond_type shift_bond_type(bond_type curr){
 
-
-  /*
-    enum bond_type {
-    
-    LEGAME_SINGOLO                = 0,
-    LEGAME_DOPPIO                 = 1,
-    LEGAME_DOPPIO_1               = 2,
-    LEGAME_DOPPIO_2               = 3,
-    LEGAME_TRIPLO                 = 4,
-    LEGAME_INTERNO                = 5,
-    LEGAME_INTERNO_OPP            = -5,
-    LEGAME_ESTERNO                = 6,
-    LEGAME_ESTERNO_OPP            = -6,
-    LEGAME_ISPESSITO              = 7,
-    LEGAME_TRATTEGGIATO           = 8,
-    LEGAME_DOPPIO_TRATTEGGIATO    = 9,
-    LEGAME_STEREO_SCON            = 10 // keep always last!!!!
-    };
-     
-    #define LAST_BOND               LEGAME_STEREO_SCON
-  */
-
   int curr_int=static_cast<int>(curr);
   int res=LEGAME_SINGOLO;
 
@@ -1878,3 +1805,156 @@ bool guess_is_cml_file(std::string f){
   return res;
 
 }
+
+bool is_ELF_shared_object(std::string file){
+  std::string sub=file.substr(file.size()-2,2);
+  if(sub=="so"){
+    return true;
+  }else{
+    return false;
+  }
+
+}
+
+proc_bezier* add_bezier_along_circle(gruppo* grp, float center_x, float center_y, float radius , float t1, float t2){
+  //from http://www.tinaja.com/glib/bezcirc2.pdf
+
+  std::pair<float, float> v1(0,0);
+  std::pair<float, float> v2(0,0);
+  std::pair<float, float> v1_tg(0,0);
+  std::pair<float, float> v2_tg(0,0);
+
+  float theta=t2-t1;
+  v1.first = cos(theta/2.0);
+  v1.second = sin(theta/2.0);
+
+  v2.first  = v1.first;
+  v2.second = -v1.second;
+
+  v1_tg.first = (4.0-v1.first)/3.0;
+  v1_tg.second = ( ((1.0-v1.first)*(3.0-v1.first))/(3.0*v1.second) );
+
+  v2_tg.first  = v1_tg.first;
+  v2_tg.second = - v1_tg.second; 
+
+  float bezAng = -(t1 + theta/2.0);
+
+  v1=bidimensional_vector::rotate(v1,bezAng);
+  v2=bidimensional_vector::rotate(v2,bezAng);
+  v1_tg=bidimensional_vector::rotate(v1_tg,bezAng);
+  v2_tg=bidimensional_vector::rotate(v2_tg,bezAng);
+
+  v1=bidimensional_vector::scale(v1,radius);
+  v2=bidimensional_vector::scale(v2,radius);
+  v1_tg=bidimensional_vector::scale(v1_tg,radius);
+  v2_tg=bidimensional_vector::scale(v2_tg,radius);
+
+
+  v1=bidimensional_vector::translate(v1,center_x,center_y);
+  v2=bidimensional_vector::translate(v2,center_x,center_y);
+  v1_tg=bidimensional_vector::translate(v1_tg,center_x,center_y);
+  v2_tg=bidimensional_vector::translate(v2_tg,center_x,center_y);
+
+  proc_bezier* bz=new proc_bezier();
+  bz->ix(v1.first);
+  bz->iy(v1.second);
+  bz->ex(v2.first);
+  bz->ey(v2.second);
+  bz->tan1x(v1_tg.first);
+  bz->tan1y(v1_tg.second);
+  bz->dash(0);
+  bz->tan2x(v2_tg.first);
+  bz->tan2y(v2_tg.second);
+  bz->spessore(1);
+
+  int nwid=grp->aggiungi_procedura(bz);
+  return dynamic_cast<proc_bezier*>(grp->find_proc_id(nwid));
+
+}
+
+void s_regmatch(std::string regex,std::string st,
+                std::vector<std::string>& vec_res,
+                bool case_ins){
+  regex_t rex;
+  if(case_ins){
+    regcomp(&rex,regex.c_str(),REG_ICASE);
+  }else{
+    regcomp(&rex,regex.c_str(),REG_ICASE);
+  }
+
+  regmatch_t res_match;
+
+  int res=regexec (&rex, st.c_str(), 
+                   1, &res_match, 0);
+
+  if(res==0){
+    std::string match=st.substr(res_match.rm_so,res_match.rm_eo-res_match.rm_so);
+    vec_res.push_back(match);
+    regfree (&rex);
+    if(res_match.rm_eo< static_cast<int>(st.size())){
+      s_regmatch(regex,st.substr(res_match.rm_eo),vec_res,case_ins);
+    }
+  }else{
+    char err[255];
+    regerror (res, &rex, err, 255);
+    regfree (&rex);
+  }
+
+}
+
+std::string s_replace(std::string regex, std::string where,
+		      std::string what,
+		      bool case_ins){
+  regex_t rex;
+  if(case_ins){
+    regcomp(&rex,regex.c_str(),REG_ICASE);
+  }else{
+    regcomp(&rex,regex.c_str(),REG_ICASE);
+  }
+
+  regmatch_t res_match;
+
+  int res=regexec (&rex, where.c_str(), 
+                   1, &res_match, 0);
+
+  std::string result;
+  if(res==0){
+    std::string match=where.substr(res_match.rm_so,res_match.rm_eo-res_match.rm_so);
+    std::string lft=where.substr(0,res_match.rm_so);
+    std::string rght=where.substr(res_match.rm_eo);
+    result= lft + what + rght;
+    regfree (&rex);
+  }else{
+    char err[255];
+    regerror (res, &rex, err, 255);
+    regfree (&rex);
+  }
+  return result;
+}
+
+
+void dialog_position(Fl_Window* win, int offset){
+
+  int x;
+  int y;
+  int w;
+  int h;
+  int mx=Fl::event_x_root();
+  int my=Fl::event_y_root();
+  Fl::screen_xywh(x, y, w, h);
+
+  if(mx + offset + win->w() > w){
+    mx = mx -win->w()-offset;  
+  }else{
+    mx = mx + offset;  
+  }
+
+  if(my + offset + win->h() > h){
+    my = my - win->h() -offset;  
+  }else{
+    my = my + offset;  
+  }
+  
+  win->position(mx,my);
+
+}
